package ps2;

import java.util.*;
import java.io.*;

import techniques.BN.*;
import techniques.BN.solver.*;

/**
 *  Implements the variable elimination algorithm for calcluating
 *  conditional probability on Bayesian networks.
 */
public class GreedyVariableEliminationSolver extends VariableEliminationSolver {

    public static final boolean DEBUG = false;

    public GreedyVariableEliminationSolver(BayesNet _bn) {
	super(_bn);
    }

    public double[] getProb(Variable variable) {
	// find the vars we need to eliminate
	HashSet nonEvidenceVars = new HashSet();
	LinkedList nodes = this.bn.getNodes();
	for (int i = 0; i < nodes.size(); i++) 
	    if (!nodes.get(i).equals(variable) && !this.evidence.containsKey(nodes.get(i)))
		nonEvidenceVars.add((Variable)nodes.get(i));

	HashSet factors = buildInitialFactors();

	while (!nonEvidenceVars.isEmpty()) {
	    if (DEBUG) System.out.println("Remaining vars: " + nonEvidenceVars);
	    int minFactSize = Integer.MAX_VALUE;
	    Variable minFactVariable = null;
	    HashSet minFactFactors = null;

	    Iterator varIter = nonEvidenceVars.iterator();
	    while (varIter.hasNext()) {
		Variable currVar = (Variable)varIter.next();
		if (DEBUG) System.out.println("  Considering " + currVar);
		HashSet currVarFactors = new HashSet();
		HashSet currVarFactorDependents = new HashSet();

		Iterator factorIter = factors.iterator();
		while (factorIter.hasNext()) {
		    Factor currFact = (Factor)factorIter.next();
		    if (currFact.dependsOn(currVar)) {
			currVarFactors.add(currFact);
			if (DEBUG) System.out.print("    " + currFact + " depends on " + currVar + "\n      adding [");
			for (int i = 0; i < currFact.dependentVars.length; i++) {
			    if (currFact.dependentVars[i].equals(currVar)) continue;
			    if (DEBUG) System.out.print(currFact.dependentVars[i] + ",");
			    currVarFactorDependents.add(currFact.dependentVars[i]);
			}
			if (DEBUG) System.out.println("]");
		    }
		}

		// calc new factor size if this var were eliminated
		int currVarFactSize = 1;
		Iterator dependentVarIter = currVarFactorDependents.iterator();
		while (dependentVarIter.hasNext()) {
		    currVarFactSize *= ((Variable)dependentVarIter.next()).getNumValues();
		}
		if (currVarFactSize < minFactSize) {
		    if (DEBUG) System.out.println("      ==> " + currVar + " creates smaller factor (" + currVarFactSize + ") than " + minFactVariable + " (" + minFactSize + ")");
		    minFactVariable = currVar;
		    minFactFactors = currVarFactors;
		    minFactSize = currVarFactSize;
		}
	    }

	    // sum out min var!
	    nonEvidenceVars.remove(minFactVariable);
	    Iterator depFactIter = minFactFactors.iterator();
	    while (depFactIter.hasNext()) {
		factors.remove(depFactIter.next());
	    }
	    factors.add(sumOutVar(minFactVariable, minFactFactors));
	}

	return ProbUtils.normalize(calculateProbsGivenVariable(variable, factors));

    }


    public static void main(String[] args) throws Exception {
	try {
	    BufferedWriter out = null;

 	    BayesNet burg = techniques.BN.Nets.getBurglary();
 	    Solver burgEngine = new GreedyVariableEliminationSolver(burg);

   	    BayesNet ins = techniques.BN.Nets.getInsurance();
   	    GreedyVariableEliminationSolver insEngine = new GreedyVariableEliminationSolver(ins);

 	    BayesNet carpo = techniques.BN.Nets.getCarpo();
 	    GreedyVariableEliminationSolver carpoEngine = new GreedyVariableEliminationSolver(carpo);

//  	    // 1.1	    
//   	    burgEngine.setEvidenceGivenVal(Evidence.get("1.1"));
//   	    ProbUtils.printProbs(burgEngine, burg.getNode("Burglary"));

//   	    // 1.2 
//   	    insEngine.setEvidenceGivenVal(Evidence.get("1.2"));
//   	    ProbUtils.printProbs(insEngine, ins.getNode("PropCost"));

 	    // 2.a 
 	    insEngine.setEvidenceGivenVal(Evidence.get("2.a.1"));
 	    ProbUtils.printProbs(insEngine, ins.getNode("PropCost"));
 	    insEngine.setEvidenceGivenVal(Evidence.get("2.a.2"));
 	    ProbUtils.printProbs(insEngine, ins.getNode("PropCost"));

 	    // 2.b
 	    carpoEngine.setEvidenceGivenVal(Evidence.get("2.b.1"));
 	    ProbUtils.printProbs(carpoEngine, carpo.getNode("N112"));
 	    carpoEngine.setEvidenceGivenVal(Evidence.get("2.b.2"));
 	    ProbUtils.printProbs(carpoEngine, carpo.getNode("N143"));


	}
	catch (Exception e) {
	    e.printStackTrace(System.out);
	}
    }

    
}
