package ps2;

import java.util.*;
import java.io.*;

import techniques.BN.*;
import techniques.BN.solver.*;

/**
 *  Implements the likelihood weighting algorithm for calcluating
 *  conditional probability on Bayesian networks.
 */
public class LikelihoodWeightingSolver extends Solver {

    public static final boolean DEBUG = false;
    public static final int DEFAULT_N = 10000;
    
    protected int noSamples;

    public LikelihoodWeightingSolver(BayesNet _bn) {
	super(_bn);
	this.noSamples = DEFAULT_N;
    }

    public void setNoSamples(int noSamples) {
	this.noSamples = noSamples;
    }

    /**
     *  Gets a probability for the given variable, over the given
     *  number of samples.
     */
    public double[] getProb(Variable variable) {
	double[] w = new double[variable.getNumValues()];
	Variable[] nodes = (Variable[])this.bn.getNodesWithTopologicalOrdering().toArray(new Variable[0]);

	for (int i = 0; i < this.noSamples; i++) {
	    if (DEBUG && i % 1000 == 999) System.out.print(".");
	    if (DEBUG && i % 50000 == 49999) System.out.println();
	    WeightedSample ws = new WeightedSample(nodes, this.evidence);
	    w[ws.getAssignedValue(variable)] += ws.w;
	}

	return ProbUtils.normalize(w);
    }


    public static void main(String[] args) throws Exception {
	BayesNet burg = techniques.BN.Nets.getBurglary();
	LikelihoodWeightingSolver burgEngine = new LikelihoodWeightingSolver(burg);

// 	// 5.1	    
// 	burgEngine.setNoSamples(100000);
// 	burgEngine.setEvidenceGivenVal(Evidence.get("1.1"));
// 	ProbUtils.printProbs(burgEngine, burg.getNode("Burglary"));

// 	BayesNet ins = techniques.BN.Nets.getInsurance();
// 	LikelihoodWeightingSolver insEngine = new LikelihoodWeightingSolver(ins);

// 	// 5.2 
// 	insEngine.setNoSamples(100000);
// 	insEngine.setEvidenceGivenVal(Evidence.get("1.2"));
// 	ProbUtils.printProbs(insEngine, ins.getNode("PropCost"));

//  	BayesNet carpo = techniques.BN.Nets.getCarpo();
//  	LikelihoodWeightingSolver carpoEngine = new LikelihoodWeightingSolver(carpo);

//  	BufferedWriter out = null;

//  	// 6.a.1
//  	VariableEliminationSolver insVarElim = new VariableEliminationSolver(ins);
//  	insVarElim.setEvidenceGivenVal(Evidence.get("2.a.1"));
//  	double[] exact2a1 = insVarElim.getProb(ins.getNode("PropCost"));
//  	insVarElim.setEvidenceGivenVal(Evidence.get("2.a.2"));
//  	double[] exact2a2 = insVarElim.getProb(ins.getNode("PropCost"));

//  	VariableEliminationSolver carpoVarElim = new VariableEliminationSolver(carpo);
//  	carpoVarElim.setEvidenceGivenVal(Evidence.get("2.b.1"));
//  	double[] exact2b1 = carpoVarElim.getProb(carpo.getNode("N112"));
//  	carpoVarElim.setEvidenceGivenVal(Evidence.get("2.b.2"));
//  	double[] exact2b2 = carpoVarElim.getProb(carpo.getNode("N143"));
	
//   	int[] noSamples = new int[] {10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000, 100000};

//  	for (int loop = 0; loop < 10; loop++) {
//  	    out = new BufferedWriter(new FileWriter("ps2/prob6.a.1-" + loop + ".dat"));	
//  	    insEngine.setEvidenceGivenVal(Evidence.get("2.a.1"));
//  	    for (int i = 0; i < noSamples.length; i++) {
//  		try {
//  		    System.out.println("Likelihood Weighting Loop " + loop + " - Running 6.a.1 on " + noSamples[i] + " samples");
//  		    insEngine.setNoSamples(noSamples[i]);
//  		    double[] currProb = insEngine.getProb(ins.getNode("PropCost"));
//  		    out.write(noSamples[i] + "\t" + ProbUtils.kullbackLeiblerDivergence(exact2a1, currProb) + "\n");
//  		    out.flush();
//  		}
//  		catch (Exception e) {
//  		    System.out.println(">>> Exception! <<<");
//  		    e.printStackTrace(System.out);
//  		}
//  	    }
//  	    out.close();

//  	    out = new BufferedWriter(new FileWriter("ps2/prob6.a.2-" + loop + ".dat"));	
//  	    insEngine.setEvidenceGivenVal(Evidence.get("2.a.2"));
//  	    for (int i = 0; i < noSamples.length; i++) {
//  		try {
//  		    System.out.println("Likelihood Weighting Loop " + loop + " - Running 6.a.2 on " + noSamples[i] + " samples");
//  		    insEngine.setNoSamples(noSamples[i]);
//  		    double[] currProb = insEngine.getProb(ins.getNode("PropCost"));
//  		    out.write(noSamples[i] + "\t" + ProbUtils.kullbackLeiblerDivergence(exact2a2, currProb) + "\n");
//  		    out.flush();
//  		}
//  		catch (Exception e) {
//  		    System.out.println(">>> Exception! <<<");
//  		    e.printStackTrace(System.out);
//  		}
//  	    }
//  	    out.close();

//  	    out = new BufferedWriter(new FileWriter("ps2/prob6.b.1-" + loop + ".dat"));	
//  	    carpoEngine.setEvidenceGivenVal(Evidence.get("2.b.1"));
//  	    for (int i = 0; i < noSamples.length; i++) {
//  		try {
//  		    System.out.println("Likelihood Weighting Loop " + loop + " - Running 6.b.1 on " + noSamples[i] + " samples");
//  		    carpoEngine.setNoSamples(noSamples[i]);
//  		    double[] currProb = carpoEngine.getProb(carpo.getNode("N112"));
//  		    out.write(noSamples[i] + "\t" + ProbUtils.kullbackLeiblerDivergence(exact2b1, currProb) + "\n");
//  		    out.flush();
//  		}
//  		catch (Exception e) {
//  		    System.out.println(">>> Exception! <<<");
//  		    e.printStackTrace(System.out);
//  		}
//  	    }
//  	    out.close();

//  	    out = new BufferedWriter(new FileWriter("ps2/prob6.b.2-." + loop + "dat"));	
//  	    carpoEngine.setEvidenceGivenVal(Evidence.get("2.b.2"));
//  	    for (int i = 0; i < noSamples.length; i++) {
//  		try {
//  		    System.out.println("Likelihood Weighting Loop " + loop + " - Running 6.b.2 on " + noSamples[i] + " samples");
//  		    carpoEngine.setNoSamples(noSamples[i]);
//  		    double[] currProb = carpoEngine.getProb(carpo.getNode("N143"));
//  		    out.write(noSamples[i] + "\t" + ProbUtils.kullbackLeiblerDivergence(exact2b2, currProb) + "\n");
//  		    out.flush();
//  		}
//  		catch (Exception e) {
//  		    System.out.println(">>> Exception! <<<");
//  		    e.printStackTrace(System.out);
//  		}
//  	    }
//  	    out.close();

//  	}

 	BayesNet ins = techniques.BN.Nets.getInsurance();
  	BayesNet carpo = techniques.BN.Nets.getCarpo();
	VariableEliminationSolver insVarElim = new VariableEliminationSolver(ins);
 	VariableEliminationSolver carpoVarElim = new VariableEliminationSolver(carpo);
	LikelihoodWeightingSolver insEngine = new LikelihoodWeightingSolver(ins);
 	LikelihoodWeightingSolver carpoEngine = new LikelihoodWeightingSolver(carpo);
   	int[] noSamples = new int[] {10, 20, 50, 100, 110, 130, 150, 180, 200, 500, 1000, 2000, 4000, 5000, 8000, 10000};

	insVarElim.setEvidenceGivenVal(Evidence.get("2.a.1"));
	insVarElim.getTimedProb(ins.getNode("PropCost"));
	System.out.println("VarElim 2.a.1 took " + insVarElim.getLastTime() + " ms");
 	for (int i = 0; i < noSamples.length; i++) {
	    insEngine.setNoSamples(noSamples[i]);
	    insEngine.setEvidenceGivenVal(Evidence.get("2.a.1"));
 	    insEngine.getTimedProb(ins.getNode("PropCost"));
	    System.out.println("Likelihood (" + noSamples[i] + ") 2.a.1 took " + insEngine.getLastTime() + " ms");
	}

	insVarElim.setEvidenceGivenVal(Evidence.get("2.a.2"));
	insVarElim.getTimedProb(ins.getNode("PropCost"));
	System.out.println("VarElim 2.a.2 took " + insVarElim.getLastTime() + " ms");
 	for (int i = 0; i < noSamples.length; i++) {
	    insEngine.setNoSamples(noSamples[i]);
	    insEngine.setEvidenceGivenVal(Evidence.get("2.a.2"));
 	    insEngine.getTimedProb(ins.getNode("PropCost"));
	    System.out.println("Likelihood (" + noSamples[i] + ") 2.a.2 took " + insEngine.getLastTime() + " ms");
	}

	carpoVarElim.setEvidenceGivenVal(Evidence.get("2.b.1"));
	carpoVarElim.getTimedProb(carpo.getNode("N112"));
	System.out.println("VarElim 2.b.1 took " + carpoVarElim.getLastTime() + " ms");
 	for (int i = 0; i < noSamples.length; i++) {
	    carpoEngine.setNoSamples(noSamples[i]);
	    carpoEngine.setEvidenceGivenVal(Evidence.get("2.b.1"));
 	    carpoEngine.getTimedProb(carpo.getNode("N112"));
	    System.out.println("Likelihood (" + noSamples[i] + ") 2.b.1 took " + carpoEngine.getLastTime() + " ms");
	}

	carpoVarElim.setEvidenceGivenVal(Evidence.get("2.b.2"));
	carpoVarElim.getTimedProb(carpo.getNode("N143"));
	System.out.println("VarElim 2.b.2 took " + carpoVarElim.getLastTime() + " ms");
 	for (int i = 0; i < noSamples.length; i++) {
	    carpoEngine.setNoSamples(noSamples[i]);
	    carpoEngine.setEvidenceGivenVal(Evidence.get("2.b.2"));
 	    carpoEngine.getTimedProb(carpo.getNode("N143"));
	    System.out.println("Likelihood (" + noSamples[i] + ") 2.b.2 took " + carpoEngine.getLastTime() + " ms");
	}
    }
}

class WeightedSample {

    public static boolean DEBUG;

    static {
	DEBUG = LikelihoodWeightingSolver.DEBUG;
    }

    protected double w;
    protected HashMap values;

    /**
     *  Takes a weighted sample over the given nodes with the given evidence.
     *
     *  @param nodes nodes from a BayesNet, in topological order
     *  @param evidence the evidence to sample against
     */
    public WeightedSample(Variable[] nodes, HashMap evidence) {
	this.values = (HashMap)evidence.clone();
	this.w = 1; 

	for (int i = 0; i < nodes.length; i++) {
	    if (DEBUG) System.out.println("  Considering " + nodes[i] + ", current values " + this.values);
	    if (this.values.containsKey(nodes[i])) {
		this.w *= nodes[i].getProb(this.values);
		if (DEBUG) System.out.println("    ==> in evidence, got prob " + nodes[i].getProb(this.values) + ", new w = " + this.w);
	    }
	    else {
		double[] dist = new double[nodes[i].getNumValues()];
		for (int j = 0; j < dist.length; j++) {
		    HashMap currEvidence = (HashMap)this.values.clone();
		    currEvidence.put(nodes[i], new Integer(j));
		    dist[j] = nodes[i].getProb(currEvidence);
		}
		if (DEBUG) System.out.println("    " + ProbUtils.getProbString(dist));
		int sample = techniques.utils.Random.sampleCompleteProbDist(dist);
		this.values.put(nodes[i], new Integer(sample));
		if (DEBUG) System.out.println("    ==> not in evidence, sampled " + nodes[i] + "=" + sample);
	    }
	}
    }

    /**
     *  Gets the value that was assigned to the given Variable in this
     *  sample.
     */
    public int getAssignedValue(Variable v) {
	return ((Integer)this.values.get(v)).intValue();
    }

}
