package ps2;

import java.util.*;

import techniques.BN.solver.Solver;
import techniques.BN.Variable;

public class ProbUtils {

    public static final boolean DEBUG = false;

    public static double[] normalize(double[] in) {
	double sum = 0;
	if (DEBUG) System.out.print("Normalizing [");
	for (int i = 0; i < in.length; i++) {
	    sum += in[i];
	    if (DEBUG) System.out.print(in[i] + ",");
	}
	if (DEBUG) System.out.print("] to [");
	double[] normalized = new double[in.length];
	for (int i = 0; i < in.length; i++) {
	    normalized[i] = in[i] / sum;
	    if (DEBUG) System.out.print(normalized[i] + ",");
	}
	if (DEBUG) System.out.println("]");
	return normalized;
    }


    public static long printProbs(Solver engine, Variable var) {
	double[] probs = engine.getTimedProb(var);
	System.out.print("P(" + var + " | ");
	HashMap evidence = engine.getEvidence();
	Iterator evIter = evidence.keySet().iterator();
	while (evIter.hasNext()) {
	    Variable next = (Variable)evIter.next();
	    System.out.print(next + "=" + next.getGivenValue(((Integer)evidence.get(next)).intValue()));
	    if (evIter.hasNext())
		System.out.print(", ");
	}
	System.out.println("):");
	for (int i = 0; i < var.getNumValues(); i++) {
	    System.out.println("  " + var.getGivenValue(i) + " = " + probs[i]);
	}
	System.out.println("   ==> Took " + engine.getLastTime() + " ms");
	return engine.getLastTime();
    }

    public static String getProbString(double[] probs) {
	StringBuffer out = new StringBuffer();
	out.append("<");
	for (int i = 0; i < probs.length; i++) {
	    out.append(probs[i]);
	    if (i < probs.length-1)
		out.append(",");
	}
	out.append(">");
	return out.toString();
    }

    /**
     *  Computes the Kullback-Leibler Divergence of an approximate
     *  probability distribution versus the exact distribution
     */
    public static double kullbackLeiblerDivergence(double[] exact, double[] approx) {
	if (exact.length != approx.length) return Double.POSITIVE_INFINITY;
	double sum = 0;
	for (int i = 0; i < exact.length; i++) {
	    sum += exact[i] * Math.log(exact[i] / approx[i]);
	}
	return sum;
    }



}
