package techniques.BN;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Map;

/**
 * Maintains a bayes net
 * 
 * @author vineet
 *
 */
public class BayesNet implements Cloneable {

	protected HashMap netNodes = new HashMap();

	/**
	 * Returns node with given name
	 */
	public Variable getNode(String nodeName) {
		return (Variable) netNodes.get(nodeName);
	}

	/**
	 * @param node
	 */
	public void addNode(Variable node) {
		netNodes.put(node.getName(), node);
	}

	/**
	 * Returns nodes in the net sorted.
	 * 
	 * @param nodeName
	 * @return list of nodes
	 */
	public LinkedList getNodes() {
		return new LinkedList(netNodes.values());
	}

	/**
	 * Returns nodes in the net sorted in topological order from parents to 
	 * children, i.e. if possible makes all parents show up before their children.
	 * 
	 * @param nodeName
	 * @return list of nodes
	 */
	public LinkedList getNodesWithTopologicalOrdering() {
		LinkedList retVal = new LinkedList(netNodes.values());
		boolean swapped;
		do {
			swapped = false;
			ListIterator li = retVal.listIterator();
			while (li.hasNext()) {
				int currNdx = li.nextIndex();
				Variable curr = (Variable) li.next();

				Variable[] currPar = curr.getParents();
				for (int i = 0; i < currPar.length; i++) {
					int parNdx = retVal.indexOf(currPar[i]);
					if (parNdx > currNdx) {
						retVal.remove(currPar[i]);
						retVal.add(currNdx, currPar[i]);
						swapped = true;
					}
				}
				if (swapped) {
					// li has become invalid so break out and reenter
					break;
				}
			}

		}
		while (swapped == true);

		return retVal;
	}


	public int getNumOfNodes() {
		return netNodes.size();
	}

	/** Performs a shallow copy (variables are not cloned)
	 */
	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}
	/** Performs a deep copy (variables are cloned)
	 */
	public Object deepCopy() throws CloneNotSupportedException {
		BayesNet retVal = (BayesNet) super.clone();

		// recreate this.netNodes in retNodes
		HashMap retNodes = new HashMap();
		Iterator it = this.netNodes.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry iME = (Map.Entry) it.next();
			retNodes.put(iME.getKey(), ((Variable) iME.getValue()).clone());
		}

		retVal.netNodes = retNodes;
		return retVal;
	}
	
	protected void testNetwork() {
		Iterator nit = getNodes().iterator();
		double err = 0;
		while (nit.hasNext()) {
			Variable v = (Variable) nit.next();
			ProbTable fpt = new ProbTable(v.getParents());
			Iterator fptIt = fpt.getVariablesIterator();
			while (fptIt.hasNext()) {
				HashMap n = (HashMap) fptIt.next();
				double sum=0;
				for (int i=0;i<v.getNumValues();i++) {
					n.put(v,new Integer(i));
					sum = sum+v.getProb(n);
				}
				err += Math.abs(1-sum);
			}
		}
		System.err.println(err);
	}

}
