/*
 * Created on Oct 12, 2003
 *
 */
package techniques.BN;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Creates a number of different BayesNets
 * 
 * @author vineet
 */
public class Nets {
	/**
	 * Returns BayesNet as given in figure 14.2, pg494 of AIMA (2nd Edition) 
	 */	
	public static BayesNet getBurglary() {
		BayesNet bn = new BayesNet();
		bn.addNode(new Variable("Burglary"));
		// equiv.
		// bn.addNode(new Variable("Burglary", Object[] {Boolean.TRUE, Boolean.FALSE});
		bn.addNode(new Variable("Earthquake"));
		bn.addNode(new Variable("Alarm"));
		bn.addNode(new Variable("JohnCalls"));
		bn.addNode(new Variable("MaryCalls"));

		Variable t;
		t = bn.getNode("Burglary");
		t.setTrueProb(new double[] { 0.001 });
		//	   equiv.
		//	   t.setProb(Boolean.TRUE, new double[] { 0.001} );
		//	   t.setProb(Boolean.FALSE, new double[] { 1-0.001} );

		t = bn.getNode("Earthquake");
		t.setTrueProb(new double[] { 0.002 });

		t = bn.getNode("Alarm");
		t.setParents(
			new Variable[] { bn.getNode("Burglary"), bn.getNode("Earthquake")});
		t.setTrueProb(new double[] { 0.001, 0.29, 0.94, 0.95 });

		t = bn.getNode("JohnCalls");
		t.setParent(bn.getNode("Alarm"));
		t.setTrueProb(new double[] { 0.05, 0.90 });

		t = bn.getNode("MaryCalls");
		t.setParent(bn.getNode("Alarm"));
		t.setTrueProb(new double[] { 00.01, 0.70 });
		
		return bn;
	}

	/**
	 * Returns BayesNet as given by 
	 * http://www.cs.huji.ac.il/labs/compbio/Repository/Datasets/alarm/alarm.htm 
	 */	
	public static BayesNet getAlarm() {
		BayesNet bn = new BayesNet();
		
		// converted from BIF

		bn.addNode( new Variable("HISTORY", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("CVP", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("PCWP", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("HYPOVOLEMIA", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("LVEDVOLUME", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("LVFAILURE", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("STROKEVOLUME", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("ERRLOWOUTPUT", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("HRBP", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("HREKG", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("ERRCAUTER", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("HRSAT", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("INSUFFANESTH", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("ANAPHYLAXIS", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("TPR", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("EXPCO2", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("KINKEDTUBE", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("MINVOL", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("FIO2", new Object[] { "LOW", "NORMAL" }));
		bn.addNode( new Variable("PVSAT", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("SAO2", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("PAP", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("PULMEMBOLUS", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("SHUNT", new Object[] { "NORMAL", "HIGH" }));
		bn.addNode( new Variable("INTUBATION", new Object[] { "NORMAL", "ESOPHAGEAL", "ONESIDED" }));
		bn.addNode( new Variable("PRESS", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("DISCONNECT", new Object[] { "TRUE", "FALSE" }));
		bn.addNode( new Variable("MINVOLSET", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("VENTMACH", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("VENTTUBE", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("VENTLUNG", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("VENTALV", new Object[] { "ZERO", "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("ARTCO2", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("CATECHOL", new Object[] { "NORMAL", "HIGH" }));
		bn.addNode( new Variable("HR", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("CO", new Object[] { "LOW", "NORMAL", "HIGH" }));
		bn.addNode( new Variable("BP", new Object[] { "LOW", "NORMAL", "HIGH" }));
		
		Variable tmp;
		
		tmp = bn.getNode("HISTORY");
		tmp.setParents(new Variable[] { bn.getNode("LVFAILURE") });
		setRevTableValues(tmp, new double[] {0.9, 0.1, 0.01, 0.99});

		tmp = bn.getNode("CVP");
		tmp.setParents(new Variable[] { bn.getNode("LVEDVOLUME") });
		setRevTableValues(tmp, new double[] {0.95, 0.04, 0.01, 0.04, 0.95, 0.01, 0.01, 0.29, 0.7});

		tmp = bn.getNode("PCWP");
		tmp.setParents(new Variable[] { bn.getNode("LVEDVOLUME") });
		setRevTableValues(tmp, new double[] {0.95, 0.04, 0.01, 0.04, 0.95, 0.01, 0.01, 0.04, 0.95});

		tmp = bn.getNode("HYPOVOLEMIA");
		setRevTableValues(tmp, new double[] {0.2, 0.8});

		tmp = bn.getNode("LVEDVOLUME");
		tmp.setParents(new Variable[] { bn.getNode("HYPOVOLEMIA"), bn.getNode("LVFAILURE") });
		setRevTableValues(tmp, new double[] {0.95, 0.04, 0.01, 0.98, 0.01, 0.01, 0.01, 0.09, 0.9, 0.05, 0.9, 0.05});

		tmp = bn.getNode("LVFAILURE");
		setRevTableValues(tmp, new double[] {0.05, 0.95});

		tmp = bn.getNode("STROKEVOLUME");
		tmp.setParents(new Variable[] { bn.getNode("HYPOVOLEMIA"), bn.getNode("LVFAILURE") });
		setRevTableValues(tmp, new double[] {0.98, 0.01, 0.01, 0.95, 0.04, 0.01, 0.5, 0.49, 0.01, 0.05, 0.9, 0.05});

		tmp = bn.getNode("ERRLOWOUTPUT");
		setRevTableValues(tmp, new double[] {0.05, 0.95});

		tmp = bn.getNode("HRBP");
		tmp.setParents(new Variable[] { bn.getNode("ERRLOWOUTPUT"), bn.getNode("HR") });
		setRevTableValues(tmp, new double[] {0.98, 0.01, 0.01, 0.4, 0.59, 0.01, 0.3, 0.4, 0.3, 0.98, 0.01, 0.01, 0.01, 0.98, 0.01, 0.01, 0.01, 0.98});

		tmp = bn.getNode("HREKG");
		tmp.setParents(new Variable[] { bn.getNode("ERRCAUTER"), bn.getNode("HR") });
		setRevTableValues(tmp, new double[] {0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.98, 0.01, 0.01, 0.01, 0.98, 0.01, 0.01, 0.01, 0.98});

		tmp = bn.getNode("ERRCAUTER");
		setRevTableValues(tmp, new double[] {0.1, 0.9});

		tmp = bn.getNode("HRSAT");
		tmp.setParents(new Variable[] { bn.getNode("ERRCAUTER"), bn.getNode("HR") });
		setRevTableValues(tmp, new double[] {0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.33333334, 0.98, 0.01, 0.01, 0.01, 0.98, 0.01, 0.01, 0.01, 0.98});

		tmp = bn.getNode("INSUFFANESTH");
		setRevTableValues(tmp, new double[] {0.1, 0.9});

		tmp = bn.getNode("ANAPHYLAXIS");
		setRevTableValues(tmp, new double[] {0.01, 0.99});

		tmp = bn.getNode("TPR");
		tmp.setParents(new Variable[] { bn.getNode("ANAPHYLAXIS") });
		setRevTableValues(tmp, new double[] {0.98, 0.01, 0.01, 0.3, 0.4, 0.3});

		tmp = bn.getNode("EXPCO2");
		tmp.setParents(new Variable[] { bn.getNode("ARTCO2"), bn.getNode("VENTLUNG") });
		setRevTableValues(tmp, new double[] {0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97});

		tmp = bn.getNode("KINKEDTUBE");
		setRevTableValues(tmp, new double[] {0.04, 0.96});

		tmp = bn.getNode("MINVOL");
		tmp.setParents(new Variable[] { bn.getNode("INTUBATION"), bn.getNode("VENTLUNG") });
		setRevTableValues(tmp, new double[] {0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.6, 0.38, 0.01, 0.01, 0.5, 0.48, 0.01, 0.01, 0.5, 0.48, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97});

		tmp = bn.getNode("FIO2");
		setRevTableValues(tmp, new double[] {0.05, 0.95});

		tmp = bn.getNode("PVSAT");
		tmp.setParents(new Variable[] { bn.getNode("FIO2"), bn.getNode("VENTALV") });
		setRevTableValues(tmp, new double[] {1.0, 0.0, 0.0, 0.99, 0.01, 0.0, 0.95, 0.04, 0.01, 0.95, 0.04, 0.01, 1.0, 0.0, 0.0, 0.95, 0.04, 0.01, 0.01, 0.95, 0.04, 0.01, 0.01, 0.98});

		tmp = bn.getNode("SAO2");
		tmp.setParents(new Variable[] { bn.getNode("PVSAT"), bn.getNode("SHUNT") });
		setRevTableValues(tmp, new double[] {0.98, 0.01, 0.01, 0.01, 0.98, 0.01, 0.01, 0.01, 0.98, 0.98, 0.01, 0.01, 0.98, 0.01, 0.01, 0.69, 0.3, 0.01});

		tmp = bn.getNode("PAP");
		tmp.setParents(new Variable[] { bn.getNode("PULMEMBOLUS") });
		setRevTableValues(tmp, new double[] {0.01, 0.19, 0.8, 0.05, 0.9, 0.05});

		tmp = bn.getNode("PULMEMBOLUS");
		setRevTableValues(tmp, new double[] {0.01, 0.99});

		tmp = bn.getNode("SHUNT");
		tmp.setParents(new Variable[] { bn.getNode("INTUBATION"), bn.getNode("PULMEMBOLUS") });
		setRevTableValues(tmp, new double[] {0.1, 0.9, 0.1, 0.9, 0.01, 0.99, 0.95, 0.05, 0.95, 0.05, 0.05, 0.95});

		tmp = bn.getNode("INTUBATION");
		setRevTableValues(tmp, new double[] {0.92, 0.03, 0.05});

		tmp = bn.getNode("PRESS");
		tmp.setParents(new Variable[] { bn.getNode("INTUBATION"), bn.getNode("KINKEDTUBE"), bn.getNode("VENTTUBE") });
		setRevTableValues(tmp, new double[] {0.97, 0.01, 0.01, 0.01, 0.01, 0.3, 0.49, 0.2, 0.01, 0.01, 0.08, 0.9, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.1, 0.84, 0.05, 0.01, 0.05, 0.25, 0.25, 0.45, 0.01, 0.15, 0.25, 0.59, 0.97, 0.01, 0.01, 0.01, 0.01, 0.29, 0.3, 0.4, 0.01, 0.01, 0.08, 0.9, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.4, 0.58, 0.01, 0.01, 0.2, 0.75, 0.04, 0.01, 0.2, 0.7, 0.09, 0.01, 0.97, 0.01, 0.01, 0.01, 0.010000001, 0.90000004, 0.080000006, 0.010000001, 0.01, 0.01, 0.38, 0.6, 0.01, 0.01, 0.01, 0.97});

		tmp = bn.getNode("DISCONNECT");
		setRevTableValues(tmp, new double[] {0.1, 0.9});

		tmp = bn.getNode("MINVOLSET");
		setRevTableValues(tmp, new double[] {0.05, 0.9, 0.05});

		tmp = bn.getNode("VENTMACH");
		tmp.setParents(new Variable[] { bn.getNode("MINVOLSET") });
		setRevTableValues(tmp, new double[] {0.05, 0.93, 0.01, 0.01, 0.05, 0.01, 0.93, 0.01, 0.05, 0.01, 0.01, 0.93});

		tmp = bn.getNode("VENTTUBE");
		tmp.setParents(new Variable[] { bn.getNode("DISCONNECT"), bn.getNode("VENTMACH") });
		setRevTableValues(tmp, new double[] {0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97});

		tmp = bn.getNode("VENTLUNG");
		tmp.setParents(new Variable[] { bn.getNode("INTUBATION"), bn.getNode("KINKEDTUBE"), bn.getNode("VENTTUBE") });
		setRevTableValues(tmp, new double[] {0.97, 0.01, 0.01, 0.01, 0.95000005, 0.030000001, 0.010000001, 0.010000001, 0.4, 0.58, 0.01, 0.01, 0.3, 0.68, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.95000005, 0.030000001, 0.010000001, 0.010000001, 0.5, 0.48, 0.01, 0.01, 0.3, 0.68, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97});

		tmp = bn.getNode("VENTALV");
		tmp.setParents(new Variable[] { bn.getNode("INTUBATION"), bn.getNode("VENTLUNG") });
		setRevTableValues(tmp, new double[] {0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.01, 0.01, 0.01, 0.01, 0.97, 0.97, 0.01, 0.01, 0.01, 0.030000001, 0.95000005, 0.010000001, 0.010000001, 0.01, 0.94, 0.04, 0.01, 0.01, 0.88, 0.1, 0.01});

		tmp = bn.getNode("ARTCO2");
		tmp.setParents(new Variable[] { bn.getNode("VENTALV") });
		setRevTableValues(tmp, new double[] {0.01, 0.01, 0.98, 0.01, 0.01, 0.98, 0.04, 0.92, 0.04, 0.9, 0.09, 0.01});

		tmp = bn.getNode("CATECHOL");
		tmp.setParents(new Variable[] { bn.getNode("ARTCO2"), bn.getNode("INSUFFANESTH"), bn.getNode("SAO2"), bn.getNode("TPR") });
		setRevTableValues(tmp, new double[] {0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.05, 0.95, 0.05, 0.95, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.01, 0.99, 0.05, 0.95, 0.05, 0.95, 0.01, 0.99, 0.05, 0.95, 0.05, 0.95, 0.01, 0.99, 0.05, 0.95, 0.05, 0.95, 0.01, 0.99, 0.05, 0.95, 0.05, 0.95, 0.01, 0.99, 0.05, 0.95, 0.05, 0.95, 0.01, 0.99, 0.7, 0.3, 0.7, 0.3, 0.1, 0.9, 0.7, 0.3, 0.7, 0.3, 0.1, 0.9, 0.7, 0.3, 0.7, 0.3, 0.1, 0.9, 0.95, 0.05, 0.99, 0.01, 0.3, 0.7, 0.95, 0.05, 0.99, 0.01, 0.3, 0.7, 0.95, 0.05, 0.99, 0.01, 0.3, 0.7});

		tmp = bn.getNode("HR");
		tmp.setParents(new Variable[] { bn.getNode("CATECHOL") });
		setRevTableValues(tmp, new double[] {0.05, 0.9, 0.05, 0.01, 0.09, 0.9});

		tmp = bn.getNode("CO");
		tmp.setParents(new Variable[] { bn.getNode("HR"), bn.getNode("STROKEVOLUME") });
		setRevTableValues(tmp, new double[] {0.98, 0.01, 0.01, 0.95, 0.04, 0.01, 0.8, 0.19, 0.01, 0.95, 0.04, 0.01, 0.04, 0.95, 0.01, 0.01, 0.04, 0.95, 0.3, 0.69, 0.01, 0.01, 0.3, 0.69, 0.01, 0.01, 0.98});

		tmp = bn.getNode("BP");
		tmp.setParents(new Variable[] { bn.getNode("CO"), bn.getNode("TPR") });
		setRevTableValues(tmp, new double[] {0.98, 0.01, 0.01, 0.98, 0.01, 0.01, 0.9, 0.09, 0.01, 0.98, 0.01, 0.01, 0.1, 0.85, 0.05, 0.05, 0.2, 0.75, 0.3, 0.6, 0.1, 0.05, 0.4, 0.55, 0.01, 0.09, 0.9});

		return bn;
	}

	/**
	 * Returns BayesNet as given by 
	 * http://www.cs.huji.ac.il/labs/compbio/Repository/Datasets/insurance/insurance.htm 
	 */	
	public static BayesNet getInsurance() {
		BayesNet bn = new BayesNet();
		
		// converted from BIF

		bn.addNode( new Variable("GoodStudent", new Object[] { "True", "False" }));
		bn.addNode( new Variable("Age", new Object[] { "Adolescent", "Adult", "Senior" }));
		bn.addNode( new Variable("SocioEcon", new Object[] { "Prole", "Middle", "UpperMiddle", "Wealthy" }));
		bn.addNode( new Variable("RiskAversion", new Object[] { "Psychopath", "Adventurous", "Normal", "Cautious" }));
		bn.addNode( new Variable("VehicleYear", new Object[] { "Current", "Older" }));
		bn.addNode( new Variable("ThisCarDam", new Object[] { "None", "Mild", "Moderate", "Severe" }));
		bn.addNode( new Variable("RuggedAuto", new Object[] { "EggShell", "Football", "Tank" }));
		bn.addNode( new Variable("Accident", new Object[] { "None", "Mild", "Moderate", "Severe" }));
		bn.addNode( new Variable("MakeModel", new Object[] { "SportsCar", "Economy", "FamilySedan", "Luxury", "SuperLuxury" }));
		bn.addNode( new Variable("DrivQuality", new Object[] { "Poor", "Normal", "Excellent" }));
		bn.addNode( new Variable("Mileage", new Object[] { "FiveThou", "TwentyThou", "FiftyThou", "Domino" }));
		bn.addNode( new Variable("Antilock", new Object[] { "True", "False" }));
		bn.addNode( new Variable("DrivingSkill", new Object[] { "SubStandard", "Normal", "Expert" }));
		bn.addNode( new Variable("SeniorTrain", new Object[] { "True", "False" }));
		bn.addNode( new Variable("ThisCarCost", new Object[] { "Thousand", "TenThou", "HundredThou", "Million" }));
		bn.addNode( new Variable("Theft", new Object[] { "True", "False" }));
		bn.addNode( new Variable("CarValue", new Object[] { "FiveThou", "TenThou", "TwentyThou", "FiftyThou", "Million" }));
		bn.addNode( new Variable("HomeBase", new Object[] { "Secure", "City", "Suburb", "Rural" }));
		bn.addNode( new Variable("AntiTheft", new Object[] { "True", "False" }));
		bn.addNode( new Variable("PropCost", new Object[] { "Thousand", "TenThou", "HundredThou", "Million" }));
		bn.addNode( new Variable("OtherCarCost", new Object[] { "Thousand", "TenThou", "HundredThou", "Million" }));
		bn.addNode( new Variable("OtherCar", new Object[] { "True", "False" }));
		bn.addNode( new Variable("MedCost", new Object[] { "Thousand", "TenThou", "HundredThou", "Million" }));
		bn.addNode( new Variable("Cushioning", new Object[] { "Poor", "Fair", "Good", "Excellent" }));
		bn.addNode( new Variable("Airbag", new Object[] { "True", "False" }));
		bn.addNode( new Variable("ILiCost", new Object[] { "Thousand", "TenThou", "HundredThou", "Million" }));
		bn.addNode( new Variable("DrivHist", new Object[] { "Zero", "One", "Many" }));

		
		Variable tmp;
		
		tmp = bn.getNode ("Age");
		setRevTableValues(tmp, new double[] {  0.2, 0.6, 0.2});

		tmp = bn.getNode ("Mileage");
		setRevTableValues(tmp, new double[] {  0.1, 0.4, 0.4, 0.1});
		
		
		tmp = bn.getNode("GoodStudent"); 
		tmp.setParents(new Variable[] { bn.getNode("SocioEcon"), bn.getNode("Age") });
		setBIFValues(tmp, new Object[] {"Prole", "Adolescent"}, new double[] { 0.1, 0.9});
		setBIFValues(tmp, new Object[] {"Prole", "Adult"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Prole", "Senior"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Middle", "Adolescent"}, new double[] { 0.2, 0.8});
		setBIFValues(tmp, new Object[] {"Middle", "Adult"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Middle", "Senior"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Adolescent"}, new double[] { 0.5, 0.5});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Adult"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Senior"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Wealthy", "Adolescent"}, new double[] { 0.4, 0.6});
		setBIFValues(tmp, new Object[] {"Wealthy", "Adult"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Wealthy", "Senior"}, new double[] { 0, 1});

		tmp = bn.getNode("SocioEcon"); 
		tmp.setParents(new Variable[] { bn.getNode("Age") });
		setBIFValues(tmp, new Object[] {"Adolescent"}, new double[] { 0.4, 0.4, 0.19, 0.01});
		setBIFValues(tmp, new Object[] {"Adult"}, new double[] { 0.4, 0.4, 0.19, 0.01});
		setBIFValues(tmp, new Object[] {"Senior"}, new double[] { 0.5, 0.2, 0.29, 0.01});

		tmp = bn.getNode("RiskAversion"); 
		tmp.setParents(new Variable[] { bn.getNode("Age"), bn.getNode("SocioEcon") });
		setBIFValues(tmp, new Object[] {"Adolescent", "Prole"}, new double[] { 0.02, 0.58, 0.3, 0.1});
		setBIFValues(tmp, new Object[] {"Adolescent", "Middle"}, new double[] { 0.02, 0.38, 0.5, 0.1});
		setBIFValues(tmp, new Object[] {"Adolescent", "UpperMiddle"}, new double[] { 0.02, 0.48, 0.4, 0.1});
		setBIFValues(tmp, new Object[] {"Adolescent", "Wealthy"}, new double[] { 0.02, 0.58, 0.3, 0.1});
		setBIFValues(tmp, new Object[] {"Adult", "Prole"}, new double[] { 0.015, 0.285, 0.5, 0.2});
		setBIFValues(tmp, new Object[] {"Adult", "Middle"}, new double[] { 0.015, 0.185, 0.6, 0.2});
		setBIFValues(tmp, new Object[] {"Adult", "UpperMiddle"}, new double[] { 0.015, 0.285, 0.5, 0.2});
		setBIFValues(tmp, new Object[] {"Adult", "Wealthy"}, new double[] { 0.015, 0.285, 0.4, 0.3});
		setBIFValues(tmp, new Object[] {"Senior", "Prole"}, new double[] { 0.01, 0.09, 0.4, 0.5});
		setBIFValues(tmp, new Object[] {"Senior", "Middle"}, new double[] { 0.01, 0.04, 0.35, 0.6});
		setBIFValues(tmp, new Object[] {"Senior", "UpperMiddle"}, new double[] { 0.01, 0.09, 0.4, 0.5});
		setBIFValues(tmp, new Object[] {"Senior", "Wealthy"}, new double[] { 0.01, 0.09, 0.4, 0.5});

		tmp = bn.getNode("VehicleYear"); 
		tmp.setParents(new Variable[] { bn.getNode("SocioEcon"), bn.getNode("RiskAversion") });
		setBIFValues(tmp, new Object[] {"Prole", "Psychopath"}, new double[] { 0.15, 0.85});
		setBIFValues(tmp, new Object[] {"Prole", "Adventurous"}, new double[] { 0.15, 0.85});
		setBIFValues(tmp, new Object[] {"Prole", "Normal"}, new double[] { 0.15, 0.85});
		setBIFValues(tmp, new Object[] {"Prole", "Cautious"}, new double[] { 0.15, 0.85});
		setBIFValues(tmp, new Object[] {"Middle", "Psychopath"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"Middle", "Adventurous"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"Middle", "Normal"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"Middle", "Cautious"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Psychopath"}, new double[] { 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Adventurous"}, new double[] { 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Normal"}, new double[] { 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Cautious"}, new double[] { 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"Wealthy", "Psychopath"}, new double[] { 0.9, 0.1});
		setBIFValues(tmp, new Object[] {"Wealthy", "Adventurous"}, new double[] { 0.9, 0.1});
		setBIFValues(tmp, new Object[] {"Wealthy", "Normal"}, new double[] { 0.9, 0.1});
		setBIFValues(tmp, new Object[] {"Wealthy", "Cautious"}, new double[] { 0.9, 0.1});

		tmp = bn.getNode("ThisCarDam"); 
		tmp.setParents(new Variable[] { bn.getNode("Accident"), bn.getNode("RuggedAuto") });
		setBIFValues(tmp, new Object[] {"None", "EggShell"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Football"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Tank"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "EggShell"}, new double[] { 0.001, 0.9, 0.098, 0.001});
		setBIFValues(tmp, new Object[] {"Mild", "Football"}, new double[] { 0.2, 0.75, 0.049999, 0.000001});
		setBIFValues(tmp, new Object[] {"Mild", "Tank"}, new double[] { 0.7, 0.29, 0.009999, 0.000001});
		setBIFValues(tmp, new Object[] {"Moderate", "EggShell"}, new double[] { 0.000001, 0.000999, 0.7, 0.299});
		setBIFValues(tmp, new Object[] {"Moderate", "Football"}, new double[] { 0.001, 0.099, 0.8, 0.1});
		setBIFValues(tmp, new Object[] {"Moderate", "Tank"}, new double[] { 0.05, 0.6, 0.3, 0.05});
		setBIFValues(tmp, new Object[] {"Severe", "EggShell"}, new double[] { 0.000001, 0.000009, 0.00009, 0.9999});
		setBIFValues(tmp, new Object[] {"Severe", "Football"}, new double[] { 0.000001, 0.000999, 0.009, 0.99});
		setBIFValues(tmp, new Object[] {"Severe", "Tank"}, new double[] { 0.05, 0.2, 0.2, 0.55});

		tmp = bn.getNode("RuggedAuto"); 
		tmp.setParents(new Variable[] { bn.getNode("MakeModel"), bn.getNode("VehicleYear") });
		setBIFValues(tmp, new Object[] {"SportsCar", "Current"}, new double[] { 0.95, 0.04, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older"}, new double[] { 0.95, 0.04, 0.01});
		setBIFValues(tmp, new Object[] {"Economy", "Current"}, new double[] { 0.5, 0.5, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Older"}, new double[] { 0.9, 0.1, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current"}, new double[] { 0.2, 0.6, 0.2});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older"}, new double[] { 0.05, 0.55, 0.4});
		setBIFValues(tmp, new Object[] {"Luxury", "Current"}, new double[] { 0.1, 0.6, 0.3});
		setBIFValues(tmp, new Object[] {"Luxury", "Older"}, new double[] { 0.1, 0.6, 0.3});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current"}, new double[] { 0.05, 0.55, 0.4});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older"}, new double[] { 0.05, 0.55, 0.4});

		tmp = bn.getNode("Accident"); 
		tmp.setParents(new Variable[] { bn.getNode("Antilock"), bn.getNode("Mileage"), bn.getNode("DrivQuality") });
		setBIFValues(tmp, new Object[] {"True", "FiveThou", "Poor"}, new double[] { 0.7, 0.2, 0.07, 0.03});
		setBIFValues(tmp, new Object[] {"True", "FiveThou", "Normal"}, new double[] { 0.99, 0.007, 0.002, 0.001});
		setBIFValues(tmp, new Object[] {"True", "FiveThou", "Excellent"}, new double[] { 0.999, 0.0007, 0.0002, 0.0001});
		setBIFValues(tmp, new Object[] {"True", "TwentyThou", "Poor"}, new double[] { 0.4, 0.3, 0.2, 0.1});
		setBIFValues(tmp, new Object[] {"True", "TwentyThou", "Normal"}, new double[] { 0.98, 0.01, 0.005, 0.005});
		setBIFValues(tmp, new Object[] {"True", "TwentyThou", "Excellent"}, new double[] { 0.995, 0.003, 0.001, 0.001});
		setBIFValues(tmp, new Object[] {"True", "FiftyThou", "Poor"}, new double[] { 0.3, 0.3, 0.2, 0.2});
		setBIFValues(tmp, new Object[] {"True", "FiftyThou", "Normal"}, new double[] { 0.97, 0.02, 0.007, 0.003});
		setBIFValues(tmp, new Object[] {"True", "FiftyThou", "Excellent"}, new double[] { 0.99, 0.007, 0.002, 0.001});
		setBIFValues(tmp, new Object[] {"True", "Domino", "Poor"}, new double[] { 0.2, 0.2, 0.3, 0.3});
		setBIFValues(tmp, new Object[] {"True", "Domino", "Normal"}, new double[] { 0.95, 0.03, 0.01, 0.01});
		setBIFValues(tmp, new Object[] {"True", "Domino", "Excellent"}, new double[] { 0.985, 0.01, 0.003, 0.002});
		setBIFValues(tmp, new Object[] {"False", "FiveThou", "Poor"}, new double[] { 0.6, 0.2, 0.1, 0.1});
		setBIFValues(tmp, new Object[] {"False", "FiveThou", "Normal"}, new double[] { 0.98, 0.01, 0.005, 0.005});
		setBIFValues(tmp, new Object[] {"False", "FiveThou", "Excellent"}, new double[] { 0.995, 0.003, 0.001, 0.001});
		setBIFValues(tmp, new Object[] {"False", "TwentyThou", "Poor"}, new double[] { 0.3, 0.2, 0.2, 0.3});
		setBIFValues(tmp, new Object[] {"False", "TwentyThou", "Normal"}, new double[] { 0.96, 0.02, 0.015, 0.005});
		setBIFValues(tmp, new Object[] {"False", "TwentyThou", "Excellent"}, new double[] { 0.99, 0.007, 0.002, 0.001});
		setBIFValues(tmp, new Object[] {"False", "FiftyThou", "Poor"}, new double[] { 0.2, 0.2, 0.2, 0.4});
		setBIFValues(tmp, new Object[] {"False", "FiftyThou", "Normal"}, new double[] { 0.95, 0.03, 0.015, 0.005});
		setBIFValues(tmp, new Object[] {"False", "FiftyThou", "Excellent"}, new double[] { 0.98, 0.01, 0.005, 0.005});
		setBIFValues(tmp, new Object[] {"False", "Domino", "Poor"}, new double[] { 0.1, 0.1, 0.3, 0.5});
		setBIFValues(tmp, new Object[] {"False", "Domino", "Normal"}, new double[] { 0.94, 0.03, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"False", "Domino", "Excellent"}, new double[] { 0.98, 0.01, 0.007, 0.003});

		tmp = bn.getNode("MakeModel"); 
		tmp.setParents(new Variable[] { bn.getNode("SocioEcon"), bn.getNode("RiskAversion") });
		setBIFValues(tmp, new Object[] {"Prole", "Psychopath"}, new double[] { 0.1, 0.7, 0.2, 0, 0});
		setBIFValues(tmp, new Object[] {"Prole", "Adventurous"}, new double[] { 0.1, 0.7, 0.2, 0, 0});
		setBIFValues(tmp, new Object[] {"Prole", "Normal"}, new double[] { 0.1, 0.7, 0.2, 0, 0});
		setBIFValues(tmp, new Object[] {"Prole", "Cautious"}, new double[] { 0.1, 0.7, 0.2, 0, 0});
		setBIFValues(tmp, new Object[] {"Middle", "Psychopath"}, new double[] { 0.15, 0.2, 0.65, 0, 0});
		setBIFValues(tmp, new Object[] {"Middle", "Adventurous"}, new double[] { 0.15, 0.2, 0.65, 0, 0});
		setBIFValues(tmp, new Object[] {"Middle", "Normal"}, new double[] { 0.15, 0.2, 0.65, 0, 0});
		setBIFValues(tmp, new Object[] {"Middle", "Cautious"}, new double[] { 0.15, 0.2, 0.65, 0, 0});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Psychopath"}, new double[] { 0.2, 0.05, 0.3, 0.45, 0});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Adventurous"}, new double[] { 0.2, 0.05, 0.3, 0.45, 0});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Normal"}, new double[] { 0.2, 0.05, 0.3, 0.45, 0});
		setBIFValues(tmp, new Object[] {"UpperMiddle", "Cautious"}, new double[] { 0.2, 0.05, 0.3, 0.45, 0});
		setBIFValues(tmp, new Object[] {"Wealthy", "Psychopath"}, new double[] { 0.3, 0.01, 0.09, 0.4, 0.2});
		setBIFValues(tmp, new Object[] {"Wealthy", "Adventurous"}, new double[] { 0.3, 0.01, 0.09, 0.4, 0.2});
		setBIFValues(tmp, new Object[] {"Wealthy", "Normal"}, new double[] { 0.3, 0.01, 0.09, 0.4, 0.2});
		setBIFValues(tmp, new Object[] {"Wealthy", "Cautious"}, new double[] { 0.3, 0.01, 0.09, 0.4, 0.2});

		tmp = bn.getNode("DrivQuality"); 
		tmp.setParents(new Variable[] { bn.getNode("DrivingSkill"), bn.getNode("RiskAversion") });
		setBIFValues(tmp, new Object[] {"SubStandard", "Psychopath"}, new double[] { 1, 0, 0});
		setBIFValues(tmp, new Object[] {"SubStandard", "Adventurous"}, new double[] { 1, 0, 0});
		setBIFValues(tmp, new Object[] {"SubStandard", "Normal"}, new double[] { 1, 0, 0});
		setBIFValues(tmp, new Object[] {"SubStandard", "Cautious"}, new double[] { 1, 0, 0});
		setBIFValues(tmp, new Object[] {"Normal", "Psychopath"}, new double[] { 0.5, 0.2, 0.3});
		setBIFValues(tmp, new Object[] {"Normal", "Adventurous"}, new double[] { 0.3, 0.4, 0.3});
		setBIFValues(tmp, new Object[] {"Normal", "Normal"}, new double[] { 0, 1, 0});
		setBIFValues(tmp, new Object[] {"Normal", "Cautious"}, new double[] { 0, 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"Expert", "Psychopath"}, new double[] { 0.3, 0.2, 0.5});
		setBIFValues(tmp, new Object[] {"Expert", "Adventurous"}, new double[] { 0.01, 0.01, 0.98});
		setBIFValues(tmp, new Object[] {"Expert", "Normal"}, new double[] { 0, 0, 1});
		setBIFValues(tmp, new Object[] {"Expert", "Cautious"}, new double[] { 0, 0, 1});

		tmp = bn.getNode("Antilock"); 
		tmp.setParents(new Variable[] { bn.getNode("MakeModel"), bn.getNode("VehicleYear") });
		setBIFValues(tmp, new Object[] {"SportsCar", "Current"}, new double[] { 0.9, 0.1});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older"}, new double[] { 0.1, 0.9});
		setBIFValues(tmp, new Object[] {"Economy", "Current"}, new double[] { 0.001, 0.999});
		setBIFValues(tmp, new Object[] {"Economy", "Older"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current"}, new double[] { 0.4, 0.6});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Luxury", "Current"}, new double[] { 0.99, 0.01});
		setBIFValues(tmp, new Object[] {"Luxury", "Older"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current"}, new double[] { 0.99, 0.01});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older"}, new double[] { 0.15, 0.85});

		tmp = bn.getNode("DrivingSkill"); 
		tmp.setParents(new Variable[] { bn.getNode("Age"), bn.getNode("SeniorTrain") });
		setBIFValues(tmp, new Object[] {"Adolescent", "True"}, new double[] { 0.5, 0.45, 0.05});
		setBIFValues(tmp, new Object[] {"Adolescent", "False"}, new double[] { 0.5, 0.45, 0.05});
		setBIFValues(tmp, new Object[] {"Adult", "True"}, new double[] { 0.3, 0.6, 0.1});
		setBIFValues(tmp, new Object[] {"Adult", "False"}, new double[] { 0.3, 0.6, 0.1});
		setBIFValues(tmp, new Object[] {"Senior", "True"}, new double[] { 0.1, 0.6, 0.3});
		setBIFValues(tmp, new Object[] {"Senior", "False"}, new double[] { 0.4, 0.5, 0.1});

		tmp = bn.getNode("SeniorTrain"); 
		tmp.setParents(new Variable[] { bn.getNode("Age"), bn.getNode("RiskAversion") });
		setBIFValues(tmp, new Object[] {"Adolescent", "Psychopath"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adolescent", "Adventurous"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adolescent", "Normal"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adolescent", "Cautious"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adult", "Psychopath"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adult", "Adventurous"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adult", "Normal"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Adult", "Cautious"}, new double[] { 0, 1});
		setBIFValues(tmp, new Object[] {"Senior", "Psychopath"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"Senior", "Adventurous"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"Senior", "Normal"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"Senior", "Cautious"}, new double[] { 0.9, 0.1});

		tmp = bn.getNode("ThisCarCost"); 
		tmp.setParents(new Variable[] { bn.getNode("ThisCarDam"), bn.getNode("CarValue"), bn.getNode("Theft") });
		setBIFValues(tmp, new Object[] {"None", "FiveThou", "True"}, new double[] { 0.2, 0.8, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "FiveThou", "False"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "TenThou", "True"}, new double[] { 0.05, 0.95, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "TenThou", "False"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "TwentyThou", "True"}, new double[] { 0.04, 0.01, 0.95, 0});
		setBIFValues(tmp, new Object[] {"None", "TwentyThou", "False"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "FiftyThou", "True"}, new double[] { 0.04, 0.01, 0.95, 0});
		setBIFValues(tmp, new Object[] {"None", "FiftyThou", "False"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Million", "True"}, new double[] { 0.04, 0.01, 0.2, 0.75});
		setBIFValues(tmp, new Object[] {"None", "Million", "False"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "FiveThou", "True"}, new double[] { 0.15, 0.85, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "FiveThou", "False"}, new double[] { 0.95, 0.05, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "TenThou", "True"}, new double[] { 0.03, 0.97, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "TenThou", "False"}, new double[] { 0.95, 0.05, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "TwentyThou", "True"}, new double[] { 0.03, 0.02, 0.95, 0});
		setBIFValues(tmp, new Object[] {"Mild", "TwentyThou", "False"}, new double[] { 0.99, 0.01, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "FiftyThou", "True"}, new double[] { 0.03, 0.02, 0.95, 0});
		setBIFValues(tmp, new Object[] {"Mild", "FiftyThou", "False"}, new double[] { 0.99, 0.01, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "Million", "True"}, new double[] { 0.02, 0.03, 0.25, 0.7});
		setBIFValues(tmp, new Object[] {"Mild", "Million", "False"}, new double[] { 0.98, 0.01, 0.01, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "FiveThou", "True"}, new double[] { 0.05, 0.95, 0, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "FiveThou", "False"}, new double[] { 0.25, 0.75, 0, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "TenThou", "True"}, new double[] { 0.01, 0.99, 0, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "TenThou", "False"}, new double[] { 0.15, 0.85, 0, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "TwentyThou", "True"}, new double[] { 0.001, 0.001, 0.998, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "TwentyThou", "False"}, new double[] { 0.01, 0.01, 0.98, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "FiftyThou", "True"}, new double[] { 0.001, 0.001, 0.998, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "FiftyThou", "False"}, new double[] { 0.005, 0.005, 0.99, 0});
		setBIFValues(tmp, new Object[] {"Moderate", "Million", "True"}, new double[] { 0.001, 0.001, 0.018, 0.98});
		setBIFValues(tmp, new Object[] {"Moderate", "Million", "False"}, new double[] { 0.003, 0.003, 0.044, 0.95});
		setBIFValues(tmp, new Object[] {"Severe", "FiveThou", "True"}, new double[] { 0.03, 0.97, 0, 0});
		setBIFValues(tmp, new Object[] {"Severe", "FiveThou", "False"}, new double[] { 0.05, 0.95, 0, 0});
		setBIFValues(tmp, new Object[] {"Severe", "TenThou", "True"}, new double[] { 0.000001, 0.999999, 0, 0});
		setBIFValues(tmp, new Object[] {"Severe", "TenThou", "False"}, new double[] { 0.01, 0.99, 0, 0});
		setBIFValues(tmp, new Object[] {"Severe", "TwentyThou", "True"}, new double[] { 0.000001, 0.000001, 0.999998, 0});
		setBIFValues(tmp, new Object[] {"Severe", "TwentyThou", "False"}, new double[] { 0.005, 0.005, 0.99, 0});
		setBIFValues(tmp, new Object[] {"Severe", "FiftyThou", "True"}, new double[] { 0.000001, 0.000001, 0.999998, 0});
		setBIFValues(tmp, new Object[] {"Severe", "FiftyThou", "False"}, new double[] { 0.001, 0.001, 0.998, 0});
		setBIFValues(tmp, new Object[] {"Severe", "Million", "True"}, new double[] { 0.000001, 0.000001, 0.009998, 0.99});
		setBIFValues(tmp, new Object[] {"Severe", "Million", "False"}, new double[] { 0.000001, 0.000001, 0.029998, 0.97});

		tmp = bn.getNode("Theft"); 
		tmp.setParents(new Variable[] { bn.getNode("AntiTheft"), bn.getNode("HomeBase"), bn.getNode("CarValue") });
		setBIFValues(tmp, new Object[] {"True", "Secure", "FiveThou"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"True", "Secure", "TenThou"}, new double[] { 0.000002, 0.999998});
		setBIFValues(tmp, new Object[] {"True", "Secure", "TwentyThou"}, new double[] { 0.000003, 0.999997});
		setBIFValues(tmp, new Object[] {"True", "Secure", "FiftyThou"}, new double[] { 0.000002, 0.999998});
		setBIFValues(tmp, new Object[] {"True", "Secure", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"True", "City", "FiveThou"}, new double[] { 0.0005, 0.9995});
		setBIFValues(tmp, new Object[] {"True", "City", "TenThou"}, new double[] { 0.002, 0.998});
		setBIFValues(tmp, new Object[] {"True", "City", "TwentyThou"}, new double[] { 0.005, 0.995});
		setBIFValues(tmp, new Object[] {"True", "City", "FiftyThou"}, new double[] { 0.005, 0.995});
		setBIFValues(tmp, new Object[] {"True", "City", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"True", "Suburb", "FiveThou"}, new double[] { 0.00001, 0.99999});
		setBIFValues(tmp, new Object[] {"True", "Suburb", "TenThou"}, new double[] { 0.0001, 0.9999});
		setBIFValues(tmp, new Object[] {"True", "Suburb", "TwentyThou"}, new double[] { 0.0003, 0.9997});
		setBIFValues(tmp, new Object[] {"True", "Suburb", "FiftyThou"}, new double[] { 0.0003, 0.9997});
		setBIFValues(tmp, new Object[] {"True", "Suburb", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"True", "Rural", "FiveThou"}, new double[] { 0.00001, 0.99999});
		setBIFValues(tmp, new Object[] {"True", "Rural", "TenThou"}, new double[] { 0.00002, 0.99998});
		setBIFValues(tmp, new Object[] {"True", "Rural", "TwentyThou"}, new double[] { 0.00005, 0.99995});
		setBIFValues(tmp, new Object[] {"True", "Rural", "FiftyThou"}, new double[] { 0.00005, 0.99995});
		setBIFValues(tmp, new Object[] {"True", "Rural", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"False", "Secure", "FiveThou"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"False", "Secure", "TenThou"}, new double[] { 0.000002, 0.999998});
		setBIFValues(tmp, new Object[] {"False", "Secure", "TwentyThou"}, new double[] { 0.000003, 0.999997});
		setBIFValues(tmp, new Object[] {"False", "Secure", "FiftyThou"}, new double[] { 0.000002, 0.999998});
		setBIFValues(tmp, new Object[] {"False", "Secure", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"False", "City", "FiveThou"}, new double[] { 0.001, 0.999});
		setBIFValues(tmp, new Object[] {"False", "City", "TenThou"}, new double[] { 0.005, 0.995});
		setBIFValues(tmp, new Object[] {"False", "City", "TwentyThou"}, new double[] { 0.01, 0.99});
		setBIFValues(tmp, new Object[] {"False", "City", "FiftyThou"}, new double[] { 0.01, 0.99});
		setBIFValues(tmp, new Object[] {"False", "City", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"False", "Suburb", "FiveThou"}, new double[] { 0.00001, 0.99999});
		setBIFValues(tmp, new Object[] {"False", "Suburb", "TenThou"}, new double[] { 0.0002, 0.9998});
		setBIFValues(tmp, new Object[] {"False", "Suburb", "TwentyThou"}, new double[] { 0.0005, 0.9995});
		setBIFValues(tmp, new Object[] {"False", "Suburb", "FiftyThou"}, new double[] { 0.0005, 0.9995});
		setBIFValues(tmp, new Object[] {"False", "Suburb", "Million"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"False", "Rural", "FiveThou"}, new double[] { 0.00001, 0.99999});
		setBIFValues(tmp, new Object[] {"False", "Rural", "TenThou"}, new double[] { 0.0001, 0.9999});
		setBIFValues(tmp, new Object[] {"False", "Rural", "TwentyThou"}, new double[] { 0.0002, 0.9998});
		setBIFValues(tmp, new Object[] {"False", "Rural", "FiftyThou"}, new double[] { 0.0002, 0.9998});
		setBIFValues(tmp, new Object[] {"False", "Rural", "Million"}, new double[] { 0.000001, 0.999999});

		tmp = bn.getNode("CarValue"); 
		tmp.setParents(new Variable[] { bn.getNode("MakeModel"), bn.getNode("VehicleYear"), bn.getNode("Mileage") });
		setBIFValues(tmp, new Object[] {"SportsCar", "Current", "FiveThou"}, new double[] { 0, 0.1, 0.8, 0.09, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Current", "TwentyThou"}, new double[] { 0, 0.1, 0.8, 0.09, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Current", "FiftyThou"}, new double[] { 0, 0.1, 0.8, 0.09, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Current", "Domino"}, new double[] { 0, 0.1, 0.8, 0.09, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older", "FiveThou"}, new double[] { 0.03, 0.3, 0.6, 0.06, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older", "TwentyThou"}, new double[] { 0.16, 0.5, 0.3, 0.03, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older", "FiftyThou"}, new double[] { 0.4, 0.47, 0.1, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older", "Domino"}, new double[] { 0.9, 0.06, 0.02, 0.01, 0.01});
		setBIFValues(tmp, new Object[] {"Economy", "Current", "FiveThou"}, new double[] { 0.1, 0.8, 0.1, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Current", "TwentyThou"}, new double[] { 0.1, 0.8, 0.1, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Current", "FiftyThou"}, new double[] { 0.1, 0.8, 0.1, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Current", "Domino"}, new double[] { 0.1, 0.8, 0.1, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Older", "FiveThou"}, new double[] { 0.25, 0.7, 0.05, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Older", "TwentyThou"}, new double[] { 0.7, 0.2999, 0.0001, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Older", "FiftyThou"}, new double[] { 0.99, 0.009999, 0.000001, 0, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Older", "Domino"}, new double[] { 0.999998, 0.000001, 0.000001, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current", "FiveThou"}, new double[] { 0, 0.1, 0.9, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current", "TwentyThou"}, new double[] { 0, 0.1, 0.9, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current", "FiftyThou"}, new double[] { 0, 0.1, 0.9, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current", "Domino"}, new double[] { 0, 0.1, 0.9, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older", "FiveThou"}, new double[] { 0.2, 0.3, 0.5, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older", "TwentyThou"}, new double[] { 0.5, 0.3, 0.2, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older", "FiftyThou"}, new double[] { 0.7, 0.2, 0.1, 0, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older", "Domino"}, new double[] { 0.99, 0.009999, 0.000001, 0, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Current", "FiveThou"}, new double[] { 0, 0, 0, 1, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Current", "TwentyThou"}, new double[] { 0, 0, 0, 1, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Current", "FiftyThou"}, new double[] { 0, 0, 0, 1, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Current", "Domino"}, new double[] { 0, 0, 0, 1, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Older", "FiveThou"}, new double[] { 0.01, 0.09, 0.2, 0.7, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Older", "TwentyThou"}, new double[] { 0.05, 0.15, 0.3, 0.5, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Older", "FiftyThou"}, new double[] { 0.1, 0.3, 0.3, 0.3, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Older", "Domino"}, new double[] { 0.2, 0.2, 0.3, 0.3, 0});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current", "FiveThou"}, new double[] { 0, 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current", "TwentyThou"}, new double[] { 0, 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current", "FiftyThou"}, new double[] { 0, 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current", "Domino"}, new double[] { 0, 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older", "FiveThou"}, new double[] { 0.000001, 0.000001, 0.000001, 0.000001, 0.999996});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older", "TwentyThou"}, new double[] { 0.000001, 0.000001, 0.000001, 0.000001, 0.999996});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older", "FiftyThou"}, new double[] { 0.000001, 0.000001, 0.000001, 0.000001, 0.999996});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older", "Domino"}, new double[] { 0.000001, 0.000001, 0.000001, 0.000001, 0.999996});

		tmp = bn.getNode("HomeBase"); 
		tmp.setParents(new Variable[] { bn.getNode("RiskAversion"), bn.getNode("SocioEcon") });
		setBIFValues(tmp, new Object[] {"Psychopath", "Prole"}, new double[] { 0.000001, 0.8, 0.049999, 0.15});
		setBIFValues(tmp, new Object[] {"Psychopath", "Middle"}, new double[] { 0.15, 0.8, 0.04, 0.01});
		setBIFValues(tmp, new Object[] {"Psychopath", "UpperMiddle"}, new double[] { 0.35, 0.6, 0.04, 0.01});
		setBIFValues(tmp, new Object[] {"Psychopath", "Wealthy"}, new double[] { 0.489999, 0.5, 0.000001, 0.01});
		setBIFValues(tmp, new Object[] {"Adventurous", "Prole"}, new double[] { 0.000001, 0.8, 0.05, 0.149999});
		setBIFValues(tmp, new Object[] {"Adventurous", "Middle"}, new double[] { 0.01, 0.25, 0.6, 0.14});
		setBIFValues(tmp, new Object[] {"Adventurous", "UpperMiddle"}, new double[] { 0.2, 0.4, 0.3, 0.1});
		setBIFValues(tmp, new Object[] {"Adventurous", "Wealthy"}, new double[] { 0.95, 0.000001, 0.000001, 0.049998});
		setBIFValues(tmp, new Object[] {"Normal", "Prole"}, new double[] { 0.000001, 0.8, 0.05, 0.149999});
		setBIFValues(tmp, new Object[] {"Normal", "Middle"}, new double[] { 0.299999, 0.000001, 0.6, 0.1});
		setBIFValues(tmp, new Object[] {"Normal", "UpperMiddle"}, new double[] { 0.5, 0.000001, 0.4, 0.099999});
		setBIFValues(tmp, new Object[] {"Normal", "Wealthy"}, new double[] { 0.85, 0.000001, 0.001, 0.148999});
		setBIFValues(tmp, new Object[] {"Cautious", "Prole"}, new double[] { 0.000001, 0.8, 0.05, 0.149999});
		setBIFValues(tmp, new Object[] {"Cautious", "Middle"}, new double[] { 0.95, 0.000001, 0.024445, 0.025554});
		setBIFValues(tmp, new Object[] {"Cautious", "UpperMiddle"}, new double[] { 0.999997, 0.000001, 0.000001, 0.000001});
		setBIFValues(tmp, new Object[] {"Cautious", "Wealthy"}, new double[] { 0.999997, 0.000001, 0.000001, 0.000001});

		tmp = bn.getNode("AntiTheft"); 
		tmp.setParents(new Variable[] { bn.getNode("RiskAversion"), bn.getNode("SocioEcon") });
		setBIFValues(tmp, new Object[] {"Psychopath", "Prole"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"Psychopath", "Middle"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"Psychopath", "UpperMiddle"}, new double[] { 0.05, 0.95});
		setBIFValues(tmp, new Object[] {"Psychopath", "Wealthy"}, new double[] { 0.5, 0.5});
		setBIFValues(tmp, new Object[] {"Adventurous", "Prole"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"Adventurous", "Middle"}, new double[] { 0.000001, 0.999999});
		setBIFValues(tmp, new Object[] {"Adventurous", "UpperMiddle"}, new double[] { 0.2, 0.8});
		setBIFValues(tmp, new Object[] {"Adventurous", "Wealthy"}, new double[] { 0.5, 0.5});
		setBIFValues(tmp, new Object[] {"Normal", "Prole"}, new double[] { 0.1, 0.9});
		setBIFValues(tmp, new Object[] {"Normal", "Middle"}, new double[] { 0.3, 0.7});
		setBIFValues(tmp, new Object[] {"Normal", "UpperMiddle"}, new double[] { 0.9, 0.1});
		setBIFValues(tmp, new Object[] {"Normal", "Wealthy"}, new double[] { 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"Cautious", "Prole"}, new double[] { 0.95, 0.05});
		setBIFValues(tmp, new Object[] {"Cautious", "Middle"}, new double[] { 0.999999, 0.000001});
		setBIFValues(tmp, new Object[] {"Cautious", "UpperMiddle"}, new double[] { 0.999999, 0.000001});
		setBIFValues(tmp, new Object[] {"Cautious", "Wealthy"}, new double[] { 0.999999, 0.000001});

		tmp = bn.getNode("PropCost"); 
		tmp.setParents(new Variable[] { bn.getNode("OtherCarCost"), bn.getNode("ThisCarCost") });
		setBIFValues(tmp, new Object[] {"Thousand", "Thousand"}, new double[] { 0.7, 0.3, 0, 0});
		setBIFValues(tmp, new Object[] {"Thousand", "TenThou"}, new double[] { 0, 0.95, 0.05, 0});
		setBIFValues(tmp, new Object[] {"Thousand", "HundredThou"}, new double[] { 0, 0, 0.98, 0.02});
		setBIFValues(tmp, new Object[] {"Thousand", "Million"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"TenThou", "Thousand"}, new double[] { 0, 0.95, 0.05, 0});
		setBIFValues(tmp, new Object[] {"TenThou", "TenThou"}, new double[] { 0, 0.6, 0.4, 0});
		setBIFValues(tmp, new Object[] {"TenThou", "HundredThou"}, new double[] { 0, 0, 0.95, 0.05});
		setBIFValues(tmp, new Object[] {"TenThou", "Million"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"HundredThou", "Thousand"}, new double[] { 0, 0, 0.98, 0.02});
		setBIFValues(tmp, new Object[] {"HundredThou", "TenThou"}, new double[] { 0, 0, 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"HundredThou", "HundredThou"}, new double[] { 0, 0, 0.6, 0.4});
		setBIFValues(tmp, new Object[] {"HundredThou", "Million"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"Million", "Thousand"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"Million", "TenThou"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"Million", "HundredThou"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"Million", "Million"}, new double[] { 0, 0, 0, 1});

		tmp = bn.getNode("OtherCarCost"); 
		tmp.setParents(new Variable[] { bn.getNode("Accident"), bn.getNode("RuggedAuto") });
		setBIFValues(tmp, new Object[] {"None", "EggShell"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Football"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Tank"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "EggShell"}, new double[] { 0.99, 0.005, 0.00499, 0.00001});
		setBIFValues(tmp, new Object[] {"Mild", "Football"}, new double[] { 0.98, 0.01, 0.009985, 0.00005});
		setBIFValues(tmp, new Object[] {"Mild", "Tank"}, new double[] { 0.95, 0.03, 0.01998, 0.00002});
		setBIFValues(tmp, new Object[] {"Moderate", "EggShell"}, new double[] { 0.6, 0.2, 0.19998, 0.00002});
		setBIFValues(tmp, new Object[] {"Moderate", "Football"}, new double[] { 0.5, 0.2, 0.29997, 0.00003});
		setBIFValues(tmp, new Object[] {"Moderate", "Tank"}, new double[] { 0.4, 0.3, 0.29996, 0.00004});
		setBIFValues(tmp, new Object[] {"Severe", "EggShell"}, new double[] { 0.2, 0.4, 0.39996, 0.00004});
		setBIFValues(tmp, new Object[] {"Severe", "Football"}, new double[] { 0.1, 0.5, 0.39994, 0.00006});
		setBIFValues(tmp, new Object[] {"Severe", "Tank"}, new double[] { 0.005, 0.55, 0.4449, 0.0001});

		tmp = bn.getNode("OtherCar"); 
		tmp.setParents(new Variable[] { bn.getNode("SocioEcon") });
		setBIFValues(tmp, new Object[] {"Prole"}, new double[] { 0.5, 0.5});
		setBIFValues(tmp, new Object[] {"Middle"}, new double[] { 0.8, 0.2});
		setBIFValues(tmp, new Object[] {"UpperMiddle"}, new double[] { 0.9, 0.1});
		setBIFValues(tmp, new Object[] {"Wealthy"}, new double[] { 0.95, 0.05});

		tmp = bn.getNode("MedCost"); 
		tmp.setParents(new Variable[] { bn.getNode("Accident"), bn.getNode("Age"), bn.getNode("Cushioning") });
		setBIFValues(tmp, new Object[] {"None", "Adolescent", "Poor"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adolescent", "Fair"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adolescent", "Good"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adolescent", "Excellent"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adult", "Poor"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adult", "Fair"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adult", "Good"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Adult", "Excellent"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Senior", "Poor"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Senior", "Fair"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Senior", "Good"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"None", "Senior", "Excellent"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild", "Adolescent", "Poor"}, new double[] { 0.96, 0.03, 0.009, 0.001});
		setBIFValues(tmp, new Object[] {"Mild", "Adolescent", "Fair"}, new double[] { 0.98, 0.019, 0.0009, 0.0001});
		setBIFValues(tmp, new Object[] {"Mild", "Adolescent", "Good"}, new double[] { 0.99, 0.0099, 0.00009, 0.00001});
		setBIFValues(tmp, new Object[] {"Mild", "Adolescent", "Excellent"}, new double[] { 0.999, 0.00099, 0.000009, 0.000001});
		setBIFValues(tmp, new Object[] {"Mild", "Adult", "Poor"}, new double[] { 0.96, 0.03, 0.009, 0.001});
		setBIFValues(tmp, new Object[] {"Mild", "Adult", "Fair"}, new double[] { 0.98, 0.019, 0.0009, 0.0001});
		setBIFValues(tmp, new Object[] {"Mild", "Adult", "Good"}, new double[] { 0.99, 0.0099, 0.00009, 0.00001});
		setBIFValues(tmp, new Object[] {"Mild", "Adult", "Excellent"}, new double[] { 0.999, 0.00099, 0.000009, 0.000001});
		setBIFValues(tmp, new Object[] {"Mild", "Senior", "Poor"}, new double[] { 0.9, 0.07, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"Mild", "Senior", "Fair"}, new double[] { 0.95, 0.04, 0.007, 0.003});
		setBIFValues(tmp, new Object[] {"Mild", "Senior", "Good"}, new double[] { 0.97, 0.025, 0.003, 0.002});
		setBIFValues(tmp, new Object[] {"Mild", "Senior", "Excellent"}, new double[] { 0.99, 0.007, 0.002, 0.001});
		setBIFValues(tmp, new Object[] {"Moderate", "Adolescent", "Poor"}, new double[] { 0.5, 0.2, 0.2, 0.1});
		setBIFValues(tmp, new Object[] {"Moderate", "Adolescent", "Fair"}, new double[] { 0.8, 0.15, 0.03, 0.02});
		setBIFValues(tmp, new Object[] {"Moderate", "Adolescent", "Good"}, new double[] { 0.95, 0.02, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"Moderate", "Adolescent", "Excellent"}, new double[] { 0.99, 0.007, 0.002, 0.001});
		setBIFValues(tmp, new Object[] {"Moderate", "Adult", "Poor"}, new double[] { 0.5, 0.2, 0.2, 0.1});
		setBIFValues(tmp, new Object[] {"Moderate", "Adult", "Fair"}, new double[] { 0.8, 0.15, 0.03, 0.02});
		setBIFValues(tmp, new Object[] {"Moderate", "Adult", "Good"}, new double[] { 0.95, 0.02, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"Moderate", "Adult", "Excellent"}, new double[] { 0.99, 0.007, 0.002, 0.001});
		setBIFValues(tmp, new Object[] {"Moderate", "Senior", "Poor"}, new double[] { 0.3, 0.3, 0.2, 0.2});
		setBIFValues(tmp, new Object[] {"Moderate", "Senior", "Fair"}, new double[] { 0.5, 0.2, 0.2, 0.1});
		setBIFValues(tmp, new Object[] {"Moderate", "Senior", "Good"}, new double[] { 0.9, 0.07, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"Moderate", "Senior", "Excellent"}, new double[] { 0.95, 0.03, 0.01, 0.01});
		setBIFValues(tmp, new Object[] {"Severe", "Adolescent", "Poor"}, new double[] { 0.3, 0.3, 0.2, 0.2});
		setBIFValues(tmp, new Object[] {"Severe", "Adolescent", "Fair"}, new double[] { 0.5, 0.2, 0.2, 0.1});
		setBIFValues(tmp, new Object[] {"Severe", "Adolescent", "Good"}, new double[] { 0.9, 0.07, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"Severe", "Adolescent", "Excellent"}, new double[] { 0.95, 0.03, 0.01, 0.01});
		setBIFValues(tmp, new Object[] {"Severe", "Adult", "Poor"}, new double[] { 0.3, 0.3, 0.2, 0.2});
		setBIFValues(tmp, new Object[] {"Severe", "Adult", "Fair"}, new double[] { 0.5, 0.2, 0.2, 0.1});
		setBIFValues(tmp, new Object[] {"Severe", "Adult", "Good"}, new double[] { 0.9, 0.07, 0.02, 0.01});
		setBIFValues(tmp, new Object[] {"Severe", "Adult", "Excellent"}, new double[] { 0.95, 0.03, 0.01, 0.01});
		setBIFValues(tmp, new Object[] {"Severe", "Senior", "Poor"}, new double[] { 0.2, 0.2, 0.3, 0.3});
		setBIFValues(tmp, new Object[] {"Severe", "Senior", "Fair"}, new double[] { 0.3, 0.3, 0.2, 0.2});
		setBIFValues(tmp, new Object[] {"Severe", "Senior", "Good"}, new double[] { 0.6, 0.3, 0.07, 0.03});
		setBIFValues(tmp, new Object[] {"Severe", "Senior", "Excellent"}, new double[] { 0.9, 0.05, 0.03, 0.02});

		tmp = bn.getNode("Cushioning"); 
		tmp.setParents(new Variable[] { bn.getNode("RuggedAuto"), bn.getNode("Airbag") });
		setBIFValues(tmp, new Object[] {"EggShell", "True"}, new double[] { 0.5, 0.3, 0.2, 0});
		setBIFValues(tmp, new Object[] {"EggShell", "False"}, new double[] { 0.7, 0.3, 0, 0});
		setBIFValues(tmp, new Object[] {"Football", "True"}, new double[] { 0, 0.1, 0.6, 0.3});
		setBIFValues(tmp, new Object[] {"Football", "False"}, new double[] { 0.1, 0.6, 0.3, 0});
		setBIFValues(tmp, new Object[] {"Tank", "True"}, new double[] { 0, 0, 0, 1});
		setBIFValues(tmp, new Object[] {"Tank", "False"}, new double[] { 0, 0, 0.7, 0.3});

		tmp = bn.getNode("Airbag"); 
		tmp.setParents(new Variable[] { bn.getNode("MakeModel"), bn.getNode("VehicleYear") });
		setBIFValues(tmp, new Object[] {"SportsCar", "Current"}, new double[] { 1, 0});
		setBIFValues(tmp, new Object[] {"SportsCar", "Older"}, new double[] { 0.1, 0.9});
		setBIFValues(tmp, new Object[] {"Economy", "Current"}, new double[] { 1, 0});
		setBIFValues(tmp, new Object[] {"Economy", "Older"}, new double[] { 0.05, 0.95});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Current"}, new double[] { 1, 0});
		setBIFValues(tmp, new Object[] {"FamilySedan", "Older"}, new double[] { 0.2, 0.8});
		setBIFValues(tmp, new Object[] {"Luxury", "Current"}, new double[] { 1, 0});
		setBIFValues(tmp, new Object[] {"Luxury", "Older"}, new double[] { 0.6, 0.4});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Current"}, new double[] { 1, 0});
		setBIFValues(tmp, new Object[] {"SuperLuxury", "Older"}, new double[] { 0.1, 0.9});

		tmp = bn.getNode("ILiCost"); 
		tmp.setParents(new Variable[] { bn.getNode("Accident") });
		setBIFValues(tmp, new Object[] {"None"}, new double[] { 1, 0, 0, 0});
		setBIFValues(tmp, new Object[] {"Mild"}, new double[] { 0.999, 0.000998, 0.000001, 0.000001});
		setBIFValues(tmp, new Object[] {"Moderate"}, new double[] { 0.9, 0.05, 0.03, 0.02});
		setBIFValues(tmp, new Object[] {"Severe"}, new double[] { 0.8, 0.1, 0.06, 0.04});

		tmp = bn.getNode("DrivHist"); 
		tmp.setParents(new Variable[] { bn.getNode("DrivingSkill"), bn.getNode("RiskAversion") });
		setBIFValues(tmp, new Object[] {"SubStandard", "Psychopath"}, new double[] { 0.001, 0.004, 0.995});
		setBIFValues(tmp, new Object[] {"SubStandard", "Adventurous"}, new double[] { 0.002, 0.008, 0.99});
		setBIFValues(tmp, new Object[] {"SubStandard", "Normal"}, new double[] { 0.03, 0.15, 0.82});
		setBIFValues(tmp, new Object[] {"SubStandard", "Cautious"}, new double[] { 0.3, 0.3, 0.4});
		setBIFValues(tmp, new Object[] {"Normal", "Psychopath"}, new double[] { 0.1, 0.3, 0.6});
		setBIFValues(tmp, new Object[] {"Normal", "Adventurous"}, new double[] { 0.5, 0.3, 0.2});
		setBIFValues(tmp, new Object[] {"Normal", "Normal"}, new double[] { 0.9, 0.07, 0.03});
		setBIFValues(tmp, new Object[] {"Normal", "Cautious"}, new double[] { 0.95, 0.04, 0.01});
		setBIFValues(tmp, new Object[] {"Expert", "Psychopath"}, new double[] { 0.3, 0.3, 0.4});
		setBIFValues(tmp, new Object[] {"Expert", "Adventurous"}, new double[] { 0.6, 0.3, 0.1});
		setBIFValues(tmp, new Object[] {"Expert", "Normal"}, new double[] { 0.99, 0.009999, 0.000001});
		setBIFValues(tmp, new Object[] {"Expert", "Cautious"}, new double[] { 0.999998, 0.000001, 0.000001});



		return bn;
	}

	/**
	 * Returns BayesNet as given by 
	 * http://www.cs.huji.ac.il/labs/compbio/Repository/Datasets/carpo/carpo.htm 
	 */	
	public static BayesNet getCarpo() {
		BayesNet bn = new BayesNet();
		
		// converted from BIF

		bn.addNode( new Variable("N5", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N9", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N19", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N20", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N27", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N29", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N34", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N38", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N26", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N28", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N40", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N41", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N57", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N42", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N63", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N64", new Object[] { "0", "1", "2", "3" }));
		bn.addNode( new Variable("N65", new Object[] { "0", "1", "2", "3" }));
		bn.addNode( new Variable("N66", new Object[] { "0", "1", "2" }));
		bn.addNode( new Variable("N67", new Object[] { "0", "1", "2", "3" }));
		bn.addNode( new Variable("N68", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N33", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N35", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N37", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N39", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N43", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N44", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N69", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N70", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N71", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N73", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N51", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N52", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N88", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N89", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N90", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N91", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N59", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N141", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N143", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N146", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N147", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N150", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N152", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N153", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N187", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N81", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N84", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N113", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N114", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N115", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N116", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N118", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N119", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N120", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N121", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N122", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N100", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N104", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N108", new Object[] { "0", "1" }));
		bn.addNode( new Variable("N112", new Object[] { "0", "1" }));
		//bn.addNode( new Variable("N117", new Object[] { "0", "1" }));

		
		Variable tmp;

		tmp = bn.getNode("N19");
		setRevTableValues(tmp, new double[] {0.9750000238 , 0.02500000037});

		tmp = bn.getNode("N27");
		setRevTableValues(tmp, new double[] {0.9499999881 , 0.05000000075});

		tmp = bn.getNode("N29");
		setRevTableValues(tmp, new double[] {0.6499999762 , 0.349999994});

		tmp = bn.getNode("N34");
		setRevTableValues(tmp, new double[] {0.9900000095 , 0.009999999776});

		tmp = bn.getNode("N69");
		setRevTableValues(tmp, new double[] {0.9700000286 , 0.02999999933});

		tmp = bn.getNode("N70");
		setRevTableValues(tmp, new double[] {0.9900000095 , 0.009999999776});

		tmp = bn.getNode("N71");
		setRevTableValues(tmp, new double[] {0.9980000257 , 0.002000000095});

		tmp = bn.getNode("N73");
		setRevTableValues(tmp, new double[] {0.9980000257 , 0.002000000095});

		tmp = bn.getNode("N143");
		setRevTableValues(tmp, new double[] {0.8999999762 , 0.1000000015});

		tmp = bn.getNode("N146");
		setRevTableValues(tmp, new double[] {0.9700000286 , 0.02999999933});

		tmp = bn.getNode("N147");
		setRevTableValues(tmp, new double[] {0.9900000095 , 0.009999999776});

		tmp = bn.getNode("N150");
		setRevTableValues(tmp, new double[] { 0.3000000119, 0.6999999881});

		tmp = bn.getNode("N152");
		setRevTableValues(tmp, new double[] {0.9900000095 , 0.009999999776});

		tmp = bn.getNode("N153");
		setRevTableValues(tmp, new double[] {0.9990000129 , 0.001000000047});

		tmp = bn.getNode ("N5");
		tmp.setParents(new Variable[] { bn.getNode("N143") ,bn.getNode("N187") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.25 , 0.75});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.5 , 0.5}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.5 , 0.5}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N9");
		tmp.setParents(new Variable[] { bn.getNode("N5") ,bn.getNode("N19") ,bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0", "0", "0"}, new double[] { 0.8999999762 , 0.05000000075 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1", "0", "0"}, new double[] { 0.1000000015 , 0.8000000119 , 0.1000000015}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0"}, new double[] { 0.05000000075 , 0.1000000015 , 0.8500000238}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1"}, new double[] { 0.1000000015 , 0.8000000119 , 0.1000000015}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N20");
		tmp.setParents(new Variable[] { bn.getNode("N27") ,bn.getNode("N29") ,bn.getNode("N51") ,bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0"}, new double[] { 0.9800000191 , 0.009999999776 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1", "0", "0", "0"}, new double[] { 0.05000000075 , 0.8500000238 , 0.1000000015}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0", "0"}, new double[] { 0.05000000075 , 0.150000006 , 0.8000000119}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1", "0"}, new double[] { 0.1299999952 , 0.01999999955 , 0.8500000238}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "1"}, new double[] { 0.1899999976 , 0.009999999776 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N38");
		tmp.setParents(new Variable[] { bn.getNode("N34") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9900000095 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.25 , 0.75}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N26");
		tmp.setParents(new Variable[] { bn.getNode("N19") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.7799999714 , 0.200000003 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.5 , 0.25 , 0.25}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N28");
		tmp.setParents(new Variable[] { bn.getNode("N29") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.7799999714 , 0.200000003 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.400000006 , 0.400000006 , 0.200000003}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N40");
		tmp.setParents(new Variable[] { bn.getNode("N5") ,bn.getNode("N19") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.9499999881 , 0.02500000037 , 0.02500000037});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.1000000015 , 0.6999999881 , 0.200000003}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.02999999933 , 0.01999999955 , 0.9499999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N41");
		tmp.setParents(new Variable[] { bn.getNode("N27") ,bn.getNode("N29") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.9800000191 , 0.009999999776 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.05000000075 , 0.8500000238 , 0.1000000015}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.05000000075 , 0.150000006 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N57");
		tmp.setParents(new Variable[] { bn.getNode("N34") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.05000000075 , 0.9499999881});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.9499999881 , 0.05000000075}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N42");
		tmp.setParents(new Variable[] { bn.getNode("N5") ,bn.getNode("N19") ,bn.getNode("N27") ,bn.getNode("N29") ,bn.getNode("N51") });
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0", "0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1", "0", "0", "0", "0"}, new double[] { 0.400000006 , 0.6000000238}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0", "0", "0"}, new double[] { 0.25 , 0.75}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1", "0", "0"}, new double[] { 0.200000003 , 0.8000000119}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "1", "0"}, new double[] { 0.400000006 , 0.6000000238}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0", "1"}, new double[] { 0.3000000119 , 0.6999999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N63");
		tmp.setParents(new Variable[] { bn.getNode("N5") ,bn.getNode("N19") ,bn.getNode("N51") ,bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0"}, new double[] { 0.9700000286 , 0.01999999955 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1", "0", "0", "0"}, new double[] { 0.8500000238 , 0.1000000015 , 0.05000000075}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0", "0"}, new double[] { 0.8500000238 , 0.1000000015 , 0.05000000075}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1", "0"}, new double[] { 0.6000000238 , 0.3000000119 , 0.1000000015}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "1"}, new double[] { 0.9300000072 , 0.05000000075 , 0.01999999955}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N64");
		tmp.setParents(new Variable[] { bn.getNode("N29") ,bn.getNode("N27") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.9789999723 , 0.009999999776 , 0.009999999776 , 0.001000000047});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.3000000119 , 0.200000003 , 0.400000006 , 0.1000000015}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.4499999881 , 0.5 , 0 , 0.05000000075}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N65");
		tmp.setParents(new Variable[] { bn.getNode("N5") ,bn.getNode("N19") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.9700000286 , 0.009999999776 , 0.009999999776 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.400000006 , 0.5 , 0.05000000075 , 0.05000000075}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.4499999881 , 0 , 0.5 , 0.05000000075}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N66");
		tmp.setParents(new Variable[] { bn.getNode("N19") ,bn.getNode("N29") ,bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0", "0", "0"}, new double[] { 0.9800000191 , 0.009999999776 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1", "0", "0"}, new double[] { 0.398999989 , 0.001000000047 , 0.6000000238}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0"}, new double[] { 0.9800000191 , 0.01999999955 , 0}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1"}, new double[] { 0.75 , 0.05000000075 , 0.200000003}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N67");
		tmp.setParents(new Variable[] { bn.getNode("N27") ,bn.getNode("N29") ,bn.getNode("N51") ,bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0"}, new double[] { 0.9800000191 , 0.009999999776 , 0.009999999776 , 0});
		setBIFValues(tmp, new Object[] {"1", "0", "0", "0"}, new double[] { 0.200000003 , 0.6000000238 , 0.1000000015 , 0.1000000015}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0", "0"}, new double[] { 0.25 , 0.6000000238 , 0.1000000015 , 0.05000000075}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1", "0"}, new double[] { 0.5 , 0.400000006 , 0.1000000015 , 0}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "1"}, new double[] { 0.75 , 0.200000003 , 0.05000000075 , 0}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N68");
		tmp.setParents(new Variable[] { bn.getNode("N34") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.3000000119 , 0.6999999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N33");
		tmp.setParents(new Variable[] { bn.getNode("N5") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9900000095 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.1000000015 , 0.8999999762}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N35");
		tmp.setParents(new Variable[] { bn.getNode("N5") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.200000003 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N37");
		tmp.setParents(new Variable[] { bn.getNode("N27") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.3000000119 , 0.6999999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N39");
		tmp.setParents(new Variable[] { bn.getNode("N19") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.3000000119 , 0.6999999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N43");
		tmp.setParents(new Variable[] { bn.getNode("N29") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.3000000119 , 0.6999999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N44");
		tmp.setParents(new Variable[] { bn.getNode("N34") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.150000006 , 0.8500000238}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N51");
		tmp.setParents(new Variable[] { bn.getNode("N70") ,bn.getNode("N73") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.05000000075 , 0.9499999881}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.200000003 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N52");
		tmp.setParents(new Variable[] { bn.getNode("N51") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.5 , 0.5}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N88");
		tmp.setParents(new Variable[] { bn.getNode("N70") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.200000003 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N89");
		tmp.setParents(new Variable[] { bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.150000006 , 0.8500000238}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N90");
		tmp.setParents(new Variable[] { bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.5 , 0.5});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.200000003 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N91");
		tmp.setParents(new Variable[] { bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.25 , 0.75}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N59");
		tmp.setParents(new Variable[] { bn.getNode("N69") ,bn.getNode("N71") });
		setBIFValues(tmp, new Object[] {"0", "0"}, new double[] { 0.9800000191 , 0.01999999955});
		setBIFValues(tmp, new Object[] {"1", "0"}, new double[] { 0.400000006 , 0.6000000238}); 
		setBIFValues(tmp, new Object[] {"0", "1"}, new double[] { 0.150000006 , 0.8500000238}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N141");
		tmp.setParents(new Variable[] { bn.getNode("N152") ,bn.getNode("N143") ,bn.getNode("N147") ,bn.getNode("N146") ,bn.getNode("N153") });
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0", "0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1", "0", "0", "0", "0"}, new double[] { 0.5 , 0.5}); 
		setBIFValues(tmp, new Object[] {"0", "1", "0", "0", "0"}, new double[] { 0.5 , 0.5}); 
		setBIFValues(tmp, new Object[] {"0", "0", "1", "0", "0"}, new double[] { 0.5 , 0.5}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "1", "0"}, new double[] { 0.5 , 0.5}); 
		setBIFValues(tmp, new Object[] {"0", "0", "0", "0", "1"}, new double[] { 0.5 , 0.5}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N187");
		tmp.setParents(new Variable[] { bn.getNode("N150") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] {  1, 0});
		setBIFValues(tmp, new Object[] {"1"}, new double[] {  0.9499999881, 0.05000000075});
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N81");
		tmp.setParents(new Variable[] { bn.getNode("N141") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9900000095 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.009999999776 , 0.9900000095}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N84");
		tmp.setParents(new Variable[] { bn.getNode("N141") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9900000095 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.009999999776 , 0.9900000095}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N113");
		tmp.setParents(new Variable[] { bn.getNode("N141") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.1000000015 , 0.8999999762}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N114");
		tmp.setParents(new Variable[] { bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8000000119 , 0.200000003});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.150000006 , 0.8500000238}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N115");
		tmp.setParents(new Variable[] { bn.getNode("N51") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.75 , 0.25});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.1000000015 , 0.8999999762}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N116");
		tmp.setParents(new Variable[] { bn.getNode("N71") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8999999762 , 0.1000000015});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.05000000075 , 0.9499999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N118");
		tmp.setParents(new Variable[] { bn.getNode("N5") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8000000119 , 0.200000003});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.02999999933 , 0.9700000286}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N119");
		tmp.setParents(new Variable[] { bn.getNode("N19") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8500000238 , 0.150000006});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.05000000075 , 0.9499999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N120");
		tmp.setParents(new Variable[] { bn.getNode("N27") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8999999762 , 0.1000000015});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.05000000075 , 0.9499999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N121");
		tmp.setParents(new Variable[] { bn.getNode("N29") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8500000238 , 0.150000006});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.05000000075 , 0.9499999881}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N122");
		tmp.setParents(new Variable[] { bn.getNode("N34") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.75 , 0.25});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.1000000015 , 0.8999999762}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N100");
		tmp.setParents(new Variable[] { bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.5 , 0.5});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.200000003 , 0.8000000119}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N104");
		tmp.setParents(new Variable[] { bn.getNode("N70") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9499999881 , 0.05000000075});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.25 , 0.75}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N108");
		tmp.setParents(new Variable[] { bn.getNode("N69") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.8999999762 , 0.1000000015});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.02999999933 , 0.9700000286}); 
		setNonNoisyTerms(tmp);

		tmp = bn.getNode ("N112");
		tmp.setParents(new Variable[] { bn.getNode("N73") });
		setBIFValues(tmp, new Object[] {"0"}, new double[] { 0.9900000095 , 0.009999999776});
		setBIFValues(tmp, new Object[] {"1"}, new double[] { 0.009999999776 , 0.9900000095}); 
		setNonNoisyTerms(tmp);


		
		


		return bn;
	}

	/**
	 * Allows setting the values in the table in reverse order.
	 * ProbTable assumes that when entering data in a table form the least
	 * significant byte (the last variable) is toggled (incremented) first.
	 * This is counter to the BIF format, so we are just going to put the
	 * data in another ProbTable (with order reversed), and then loop through
	 * querying the reversed-order-table and putting it in the correct
	 * table
	 * 
	 * @param knownValues -  the first few parameters from variables
	 * for the current prob. table
	 * @param inTableValues
	 */
	private static void setRevTableValues(Variable var, double[] inTableValues) {
		ProbTable varPT = var.getProbTable();
		Variable[] revVar = new Variable[varPT.variables.length];
		for (int i = 0; i < varPT.variables.length; i++) {
			revVar[i] = varPT.variables[varPT.variables.length - i - 1];
		}
		ProbTable revTable = new ProbTable(revVar);
		revTable.setTableValues(new HashMap(), inTableValues);
		Iterator it = varPT.getVariablesIterator();
		while (it.hasNext()) {
			HashMap entry = (HashMap) it.next();
			double val = revTable.getProb(entry);
			varPT.setProb(entry, val);
		}
	}


	/**
	 * Designed for BIF importing purposes. Assumes that inTableValues is of 
	 * length Variable.numValues
	 */
	private static void setBIFValues(
		Variable var,
		Object[] varGivenValues,
		double[] inTableValues) {

		if (inTableValues.length != var.getNumValues()) {
			System.err.println("inTableValues.length != tmp.getNumValues()");
		}

		// input 
		int[] varValues = new int[varGivenValues.length + 1];
		for (int i = 0; i < varGivenValues.length; i++) {
			varValues[i + 1] =
				var.getProbTable().getVariable(i + 1).getValNdx(
					varGivenValues[i]);
		}

		for (int i = 0; i < inTableValues.length; i++) {
			varValues[0] = i;
			var.getProbTable().setProb(varValues, inTableValues[i]);
		}
	}

	/**
	 * @param tmp
	 */
	// Note: this is the old buggy version
	private static void setNonNoisyTerms_old(Variable var) {
		ProbTable curPT = var.getProbTable();
		Iterator it = curPT.getVariablesIterator();
		while (it.hasNext()) {
			HashMap entry = (HashMap) it.next();
			Integer entryVarVal = (Integer) entry.get(var);
			double val = curPT.getProb(entry);
			if (val == -1) {
				/* Note: since the numbers are generated in order, any two 
				 * subcomponents [1111 -> 1100 0011] should already have been
				 * calculated, so let's just toggle one bit and use that as the 
				 * basis 
				 */
				// calculate it!

				Iterator entryIt = entry.entrySet().iterator();
				Variable cellVar = null;
				int cellVal = 0;
				while (entryIt.hasNext()) {
					Map.Entry iMECell = (Map.Entry) entryIt.next();
					cellVal = ((Integer) iMECell.getValue()).intValue();
					cellVar = (Variable) iMECell.getKey();
					if ((cellVal != 0) && (cellVar != var)) {
						break;
					}
				}
				HashMap otherEntry1 = new HashMap();
				for (int i = 0; i < curPT.getNumVariables(); i++) {
					otherEntry1.put(curPT.getVariable(i), new Integer(0));
				}
				otherEntry1.put(cellVar, new Integer(cellVal));
				otherEntry1.put(var, entryVarVal);
				HashMap otherEntry2 = (HashMap) entry.clone();
				otherEntry2.put(cellVar, new Integer(0));

				val =
					1
						- ((1 - curPT.getProb(otherEntry1))
							* (1 - curPT.getProb(otherEntry2)));
				curPT.setProb(entry, val);
			}
		}

		if (var.name.equals("N64")) {
			System.err.println(var);
			it = var.getProbTable().getVariablesIterator();
			while (it.hasNext()) {
				HashMap entry = (HashMap) it.next();
				System.err.println(entry + ": " + curPT.getProb(entry));
			}
		}
		
		// normalize last number in curPT to 1
		ProbTable fakePT = new ProbTable(var.getParents());
		it = fakePT.getVariablesIterator();
		while (it.hasNext()) {
			HashMap entry = (HashMap) it.next();
			double sum = 0;
			for (int i = 0; i < var.getNumValues() - 1; i++) {
				entry.put(var, new Integer(i));
				sum += curPT.getProb(entry);
			}
			entry.put(var, new Integer(var.getNumValues() - 1));
			curPT.setProb(entry, 1 - sum);
		}
		
		/**/
		if (var.name.equals("N64")) {
			System.err.println(var);
			it = var.getProbTable().getVariablesIterator();
			while (it.hasNext()) {
				HashMap entry = (HashMap) it.next();
				System.err.println(entry + ": " + curPT.getProb(entry));
			}
		}


	}

	/**
	 * @param tmp
	 */
	private static void setNonNoisyTerms(Variable var) {
		/*
		if (var.name.equals("N64")) {
			System.err.println(var);
			Iterator it = var.getProbTable().getVariablesIterator();
			while (it.hasNext()) {
				HashMap entry = (HashMap) it.next();
				System.err.println(entry + ": " + var.getProbTable().getProb(entry));
			}
		}
		*/


		ProbTable curPT = var.getProbTable();
		Iterator it = null;


		ProbTable fakePT = new ProbTable(var.getParents());
		for (int i = 0; i < var.getNumValues() - 1; i++) {
			
			it = fakePT.getVariablesIterator();
			while (it.hasNext()) {
				// check if we still need to be set
				HashMap curEntry = (HashMap) it.next();
				curEntry.put(var, new Integer(i));
				if (curPT.getProb(curEntry) != -1) {
					continue;
				}
				
				// we need to set this term
				
				// get leaky term
				HashMap termEntry = getLeakTerm(curPT);
				termEntry.put(var, new Integer(i));
				double val = curPT.getProb(termEntry);
				
				// let's iterate through curEntry and pick up the components
				curEntry.remove(var);
				Iterator varIt = curEntry.keySet().iterator();
				while (varIt.hasNext()) {
					Variable v = (Variable) varIt.next();
					termEntry = getLeakTerm(curPT);
					termEntry.put(var, new Integer(i));
					termEntry.put(v, curEntry.get(v));
					val = val * (1-curPT.getProb(termEntry));
				}
				curEntry.put(var, new Integer(i));
				curPT.setProb(curEntry, val);
			}
		}
		
		// i=var.getNumValues-1
		int i = var.getNumValues()-1;
		it = fakePT.getVariablesIterator();
		while (it.hasNext()) {
			// check if we still need to be set
			HashMap curEntry = (HashMap) it.next();
			curEntry.put(var, new Integer(i));
			if (curPT.getProb(curEntry) != -1) {
				continue;
			}
				
			// we need to set this term
				
			double val = 1;

			// let's iterate and pick up the components
			for (int j=0;j<var.getNumValues()-1;j++) {
				curEntry.put(var, new Integer(j));
				val = val - curPT.getProb(curEntry);
			}
			curEntry.put(var, new Integer(i));
			curPT.setProb(curEntry, val);
		}
		
		/*
		if (var.name.equals("N108")) {
			System.err.println(var);
			it = var.getProbTable().getVariablesIterator();
			while (it.hasNext()) {
				HashMap entry = (HashMap) it.next();
				System.err.println(entry + ": " + curPT.getProb(entry));
			}
		}
		*/


	}

	/**
	 * @param curPT
	 * @return
	 */
	private static HashMap getLeakTerm(ProbTable curPT) {
		HashMap retVal = new HashMap();
		for (int j = 0; j < curPT.getVariables().length; j++) {
			retVal.put(curPT.getVariable(j), new Integer(0));
		}
		return retVal;
	}

}
